function [output, storeESS] = BNP_VAR(data, start_trunc, lags, trainT, hsteps, numbofparts, epsilon_trunc, numb_trunc, burnin, every, fig1, run, tot_runs)

d = uiprogressdlg(fig1,'Title','Fitting Model','Message', 'Progress');

warning off

it = start_trunc - 1;

numbofchains = 7;
if ( numbofchains == 1 )
    heat = 1;
else
    heat = exp(linspace(0, log(0.95), numbofchains));
end
rhostar_PT = zeros(1, length(heat) - 1);
for i = 1:(length(heat)-1)
    rhostar_PT(i) = log(log(heat(i)/heat(i+1)));
end
order = 1:length(heat);

top_trunc = 70;

[T, p] = size(data);
K = (lags + 1) * p;
Nfactors = p;

 disp(['T = ' num2str(T)]);
 disp(['p = ' num2str(p)]);

y_lags = zeros(T - lags, p * lags);
y_cur = zeros(T - lags, p);

for i = (lags+1):T
    y_cur(i - lags, :) = data(i, :);
    
    for j = 1:lags
        y_lags(i - lags, ((j-1)*p+1):(j*p)) = data(i - j, :);
    end
end

rhoepsilon = 10^(-3);
Vepsilon = 10^(-3);

V = ones(numbofparts, top_trunc);
mu = zeros(p, numbofparts, top_trunc);
S = zeros(p, numbofparts, top_trunc);
P = zeros(K - p, K - p, numbofparts, top_trunc);
SuperP = zeros(K, K, numbofparts, top_trunc);
lambda = zeros(p, Nfactors, numbofparts, top_trunc);
phi_idio = zeros(p, numbofchains, top_trunc);
phi = zeros(p, Nfactors, numbofparts, top_trunc);
delta = zeros(Nfactors, numbofparts, top_trunc);
tau = zeros(Nfactors, numbofparts, top_trunc);
rho = zeros(Nfactors, numbofparts, top_trunc);
rho_idio = zeros(p, numbofparts, top_trunc);
prob = zeros(numbofparts, top_trunc);
loglike = zeros(numbofparts, 1);
logpost = zeros(numbofparts, 1);
M = zeros(numbofparts, 1);
logprob_marg = zeros(trainT, top_trunc, numbofparts);
logprob_joint = zeros(trainT, top_trunc, numbofparts);
Smu = zeros(p, numbofparts);

a1 = 1;
a2 = 3;
nu = 3;

mu0 = mean(data(1:trainT, :));
Sigma0 = 1.5^2 * var(data(1:trainT, :));
Salpha = 4;
Smu_alpha = 0.01;
Smu_beta = 0.01;

M_MCMC = 1 * ones(1, numbofchains);
mu_MCMC = zeros(it, p, numbofchains);
S_MCMC = zeros(it, p, numbofchains);
P_MCMC = ones(K - p, K - p, it, numbofchains);
SuperP_MCMC = zeros(K, K, it, numbofchains);
lambda_MCMC = zeros(p, Nfactors, it, numbofchains);
phi_idio_MCMC = zeros(p, it, numbofchains);
phi_MCMC = zeros(p, Nfactors, it, numbofchains);
delta_MCMC = zeros(Nfactors, it, numbofchains);
tau_MCMC = zeros(Nfactors, it, numbofchains);
rho_MCMC = zeros(Nfactors, it, numbofchains);
rho_idio_MCMC = zeros(Nfactors, it, numbofchains);
logprob_marg_MCMC = zeros(trainT, it, numbofchains);
logprob_joint_MCMC = zeros(trainT, it, numbofchains);
loglike_MCMC = zeros(1, numbofchains);
V_MCMC = zeros(it, numbofchains);
prob_MCMC = zeros(it, numbofchains);
Smu_MCMC = ones(p, numbofchains);

alpha = zeros(1, K - 1);
alpha(K - 1) = 1;
for i = (K - 2):-1:1
    alpha(i) = alpha(i + 1) + 1 / 2;
end

diffstar1 = zeros(trainT - lags, p * lags);
diffstar2 = zeros(trainT - lags, p);
for chain = 1:numbofchains
    for comp = 1:it
        check2 = 0;
        while ( check2 == 0 )
            check2 = 1;
            mu_MCMC(comp, :, chain) = (mu0' + sqrt(Sigma0)' .* randn(p, 1));
            for i = 1:p
                S_MCMC(comp, i, chain) = 1 / gamrnd(Salpha, 1 / ((Salpha - 1) * Smu_MCMC(i, chain)));
            end
            
            delta_MCMC(1, comp, chain) = gamrnd(a1, 1);
            tau_MCMC(1, comp, chain) = delta_MCMC(1, comp, chain);
            delta_MCMC(2:end, comp, chain) = gamrnd(a2, 1, Nfactors - 1, 1);
            for i = 2:Nfactors
                tau_MCMC(i, comp, chain) = tau_MCMC(i - 1, comp, chain) * delta_MCMC(i, comp, chain);
            end
            
            for i = 1:Nfactors
                for j = 1:p
                    phi_MCMC(j, i, comp, chain) = gamrnd(nu / 2, 2 / nu);
                    lambda_MCMC(j, i, comp, chain) = randn / sqrt(tau_MCMC(i, comp, chain) * phi_MCMC(j, i, comp, chain));
                end
                rho_MCMC(i, comp, chain) = 0;
                while ( (rho_MCMC(i, comp, chain) < rhoepsilon) || (rho_MCMC(i, comp, chain) > 1 - rhoepsilon) )
                    rho_MCMC(i, comp, chain) = betarnd(1, 1);
                end
            end
            for i = 1:p
                phi_idio_MCMC(i, comp, chain) = gamrnd(nu / 2, 2 / nu);
                rho_idio_MCMC(i, comp, chain) = 0;
                while ( (rho_idio_MCMC(i, comp, chain) < rhoepsilon) || (rho_idio_MCMC(i, comp, chain) > 1 - rhoepsilon) )
                    rho_idio_MCMC(i, comp, chain) = betarnd(1, 1);
                end
            end
            
            for k = 1:Nfactors
                for i = 1:(lags + 1)
                    for j = 1:(lags + 1)
                        SuperP_MCMC((i - 1) * p + (1:p), (j - 1) * p + (1:p), comp, chain) = SuperP_MCMC((i - 1) * p + (1:p), (j - 1) * p + (1:p), comp, chain) + rho_MCMC(k, comp, chain)^(abs(i - j)) * (lambda_MCMC(:, k, comp, chain) * lambda_MCMC(:, k, comp, chain)');
                    end
                end
            end
            for k = 1:p
                for i = 1:(lags + 1)
                    for j = 1:(lags + 1)
                        SuperP_MCMC((i - 1) * p + k, (j - 1) * p + k, comp, chain) = SuperP_MCMC((i - 1) * p + k, (j - 1) * p + k, comp, chain) + rho_idio_MCMC(k, comp, chain)^(abs(i - j)) / phi_idio_MCMC(k, comp, chain);
                    end
                end
            end
            P_MCMC(:, :, comp, chain) = SuperP_MCMC(1:(lags*p), 1:(lags*p), comp, chain);
            
            sumlogS = sum(log(S_MCMC(comp, :, chain)));
            for j1 = 1:lags
                diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
            end
            diffstar2(:, :) = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
            
            [logprob_marg_MCMC((lags+1):trainT, comp, chain), check] = logmvnpdf(diffstar1, zeros(1, lags * p), P_MCMC(:, :, comp, chain));
            if ( check2 == 1 )
                check2 = check;
            end
            logprob_marg_MCMC((lags+1):trainT, comp, chain) = logprob_marg_MCMC((lags+1):trainT, comp, chain) - lags * sumlogS;
            [logprob_joint_MCMC((lags+1):trainT, comp, chain), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), SuperP_MCMC(:, :, comp, chain));
            if ( check2 == 1 )
                check2 = check;
            end
            logprob_joint_MCMC((lags+1):trainT, comp, chain) = logprob_joint_MCMC((lags+1):trainT, comp, chain) - (lags + 1) * sumlogS;
        end
    end
    V_MCMC(:, chain) = betarnd(1, M_MCMC(chain), it, 1);
    for j = 1:it
        while ( (V_MCMC(j, chain) < Vepsilon) || (V_MCMC(j, chain) > 1 - Vepsilon) )
            V_MCMC(j, chain) = betarnd(1, M_MCMC(chain));
        end
    end
    
    prodV = 1;
    for j = 1:it
        prob_MCMC(j, chain) = V_MCMC(j, chain) * prodV;
        prodV = prodV * (1 - V_MCMC(j, chain));
    end
    prob_MCMC(:, chain) = prob_MCMC(:, chain) / sum(prob_MCMC(:, chain));
    
    maxstar = max(permute(logprob_joint_MCMC((lags+1):trainT, :, chain), [2 1 3]))';
    loglike_MCMC(chain) = ones(1, trainT - lags) * (log(exp(logprob_joint_MCMC((lags+1):trainT, :, chain) - maxstar * ones(1, it)) * prob_MCMC(:, chain)) + maxstar);
    maxstar = max(permute(logprob_marg_MCMC((lags+1):trainT, :, chain), [2 1 3]))';
    loglike_MCMC(chain) = loglike_MCMC(chain) - ones(1, trainT - lags) * (log(exp(logprob_marg_MCMC((lags+1):trainT, :, chain) - maxstar * ones(1, it)) * prob_MCMC(:, chain)) + maxstar);
end


for chain1 = 1:numbofchains
    for comp = 1:it
        checkSuperP_MCMC = zeros((lags + 1) * p, (lags + 1) * p);
        for k = 1:Nfactors
            for i = 1:(lags + 1)
                for j = 1:(lags + 1)
                    checkSuperP_MCMC((i - 1) * p + (1:p), (j - 1) * p + (1:p)) = checkSuperP_MCMC((i - 1) * p + (1:p), (j - 1) * p + (1:p)) + rho_MCMC(k, comp, chain1)^(abs(i - j)) * (lambda_MCMC(:, k, comp, chain1) * lambda_MCMC(:, k, comp, chain1)');
                end
            end
        end
        for k = 1:p
            for i = 1:(lags + 1)
                for j = 1:(lags + 1)
                    checkSuperP_MCMC((i - 1) * p + k, (j - 1) * p + k) = checkSuperP_MCMC((i - 1) * p + k, (j - 1) * p + k) + rho_idio_MCMC(k, comp, chain1)^(abs(i - j)) / phi_idio_MCMC(k, comp, chain1);
                end
            end
        end
        
        if ( sum(sum(abs(checkSuperP_MCMC - SuperP_MCMC(:, :, comp, chain1)) > 0.00001 ) ) > 0 )
            stop;
        end
    end
end


logmusd = log(0.000001) * ones(p, numbofchains);
loglambdasd = log(0.00001) * ones(p, Nfactors, numbofchains);
logrhosd = log(0.00001) * ones(Nfactors, numbofchains);
logrho_idiosd = log(0.00001) * ones(p, numbofchains);
logphi_idiosd = log(0.00001) * ones(p, numbofchains);
logSsd = log(0.000001) * ones(p, numbofchains);
logVsd = log(0.000001) * ones(1, numbofchains);

muaccept = zeros(p, numbofchains);
mucount = zeros(p, numbofchains);
lambdaaccept = zeros(p, Nfactors, numbofchains);
lambdacount = zeros(p, Nfactors, numbofchains);
lambdaaccept2 = zeros(1, numbofchains);
lambdacount2 = zeros(1, numbofchains);
rhoaccept = zeros(Nfactors, numbofchains);
rhocount = zeros(Nfactors, numbofchains);
rhoaccept2 = zeros(Nfactors, numbofchains);
rhocount2 = zeros(Nfactors, numbofchains);
rho_idioaccept = zeros(Nfactors, numbofchains);
rho_idiocount = zeros(Nfactors, numbofchains);
rho_idioaccept2 = zeros(Nfactors, numbofchains);
rho_idiocount2 = zeros(Nfactors, numbofchains);
phi_idioaccept = zeros(Nfactors, numbofchains);
phi_idiocount = zeros(Nfactors, numbofchains);
phi_idioaccept2 = zeros(Nfactors, numbofchains);
phi_idiocount2 = zeros(Nfactors, numbofchains);
Saccept = zeros(p, numbofchains);
Scount = zeros(p, numbofchains);
Vaccept = zeros(1, numbofchains);
Vcount = zeros(1, numbofchains);
totalaccept2 = zeros(1, numbofchains - 1);
totalcount2 = zeros(1, numbofchains - 1);
Maccept = zeros(1, numbofchains);
Mcount = zeros(1, numbofchains);
lambdasum1 = zeros(p, comp, numbofchains);
lambdasum2 = zeros(p, p, comp, numbofchains);
lambdasum3 = zeros(comp, numbofchains);
rhosum1 = zeros(p, numbofchains);
rhosum2 = zeros(p, p, numbofchains);
rhosum3 = zeros(1, numbofchains);
rho_idiosum1 = zeros(p, numbofchains);
rho_idiosum2 = zeros(p, p, numbofchains);
rho_idiosum3 = zeros(1, numbofchains);
phi_idiosum1 = zeros(p, numbofchains);
phi_idiosum2 = zeros(p, p, numbofchains);
phi_idiosum3 = zeros(1, numbofchains);

loglambdasd2 = zeros(1, numbofchains);
logrhosd2 = zeros(1, numbofchains);
logrho_idiosd2 = zeros(1, numbofchains);
logphi_idiosd2 = zeros(1, numbofchains);

numbofits = burnin + numbofparts * every;

for it2 = 1:numbofits
    
    for chain = 1:numbofchains
        
        % Update mu
        for comp = 1:it
            for i = 1:p
                
                newmu_MCMC = mu_MCMC(comp, :, chain);
                newmu_MCMC(i) = newmu_MCMC(i) + exp(logmusd(i, order(chain))) * randn;
                
                newlogprob_marg_MCMC = logprob_marg_MCMC(:, :, chain);
                newlogprob_joint_MCMC = logprob_joint_MCMC(:, :, chain);
                
                sumlogS = sum(log(S_MCMC(comp, :, chain)));
                for j1 = 1:lags
                    diffstar1(:, ((j1-1)*p+1):(j1*p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * newmu_MCMC) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                end
                diffstar2 = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * newmu_MCMC) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                
                check2 = 1;
                [newlogprob_marg_MCMC((lags+1):trainT, comp), check] = logmvnpdf(diffstar1, zeros(1, lags * p), P_MCMC(:, :, comp, chain));
                if ( check == 0 )
                    check2 = 0;
                end
                newlogprob_marg_MCMC((lags+1):trainT, comp) = newlogprob_marg_MCMC((lags+1):trainT, comp) - lags * sumlogS;
                [newlogprob_joint_MCMC((lags+1):trainT, comp), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), SuperP_MCMC(:, :, comp, chain));
                if ( check == 0 )
                    check2 = 0;
                end
                newlogprob_joint_MCMC((lags+1):trainT, comp) = newlogprob_joint_MCMC((lags+1):trainT, comp) - (lags + 1) * sumlogS;
                
                if ( check2 == 1 )
                    z1 = prob_MCMC(1:it, chain) > 0;
                    maxstar = max(newlogprob_joint_MCMC((lags+1):trainT, z1==1), [], 2);
                    newloglike_MCMC = ones(1, trainT - lags) * (log(exp(newlogprob_joint_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                    maxstar = max(newlogprob_marg_MCMC((lags+1):trainT, z1==1), [], 2);
                    newloglike_MCMC = newloglike_MCMC - ones(1, trainT - lags) * (log(exp(newlogprob_marg_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                    
                    if ( newloglike_MCMC > 12000 )
                        stop;
                    end
                    
                    logaccept = heat(order(chain)) * (newloglike_MCMC - loglike_MCMC(chain));
                    logaccept = logaccept - 0.5 * (newmu_MCMC(i) - mu0(i))^2 / Sigma0(i);
                    logaccept = logaccept + 0.5 * (mu_MCMC(comp, i, chain) - mu0(i))^2 / Sigma0(i);
                    
                    accept = 1;
                    if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                        accept = 0;
                    elseif ( logaccept < 0 )
                        accept = exp(logaccept);
                    end
                else
                    accept = 0;
                end
                
                muaccept(i, order(chain)) = muaccept(i, order(chain)) + accept;
                mucount(i, order(chain)) = mucount(i, order(chain)) + 1;
                
                if ( rand < accept )
                    mu_MCMC(comp, i, chain) = newmu_MCMC(i);
                    loglike_MCMC(chain) = newloglike_MCMC;
                    logprob_marg_MCMC(:, comp, chain) = newlogprob_marg_MCMC(:, comp);
                    logprob_joint_MCMC(:, comp, chain) = newlogprob_joint_MCMC(:, comp);
                end
                
                logmusd(i, order(chain)) = logmusd(i, order(chain)) + 1 / it2^0.55 * (accept - 0.3);
            end
        end
        
        
        z1 = prob_MCMC(1:it, chain) > 0;
        maxstar = max(logprob_joint_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = ones(1, trainT - lags) * (log(exp(logprob_joint_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        maxstar = max(logprob_marg_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = checkloglike_MCMC - ones(1, trainT - lags) * (log(exp(logprob_marg_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        
        
        if ( abs(checkloglike_MCMC - loglike_MCMC(chain)) > 10^(-3) )
            stop;
        end
        
        % Update lambda
        for comp = 1:it
            for k = 1:Nfactors
                
                if ( it2 < 50 )
                    for j = 1:p
                        newlambda_MCMC = lambda_MCMC(:, :, comp, chain);
                        newlambda_MCMC(j, k) = newlambda_MCMC(j, k) + exp(loglambdasd(j, k, order(chain))) * randn;
                        
                        newSuperP = SuperP_MCMC(:, :, comp, chain);
                        for i1 = 1:(lags + 1)
                            for j1 = 1:(lags + 1)
                                newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) - rho_MCMC(k, comp, chain)^(abs(i1 - j1)) * (lambda_MCMC(:, k, comp, chain) * lambda_MCMC(:, k, comp, chain)');
                                newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) + rho_MCMC(k, comp, chain)^(abs(i1 - j1)) * (newlambda_MCMC(:, k) * newlambda_MCMC(:, k)');
                            end
                        end
                        newP = newSuperP(1:(lags*p), 1:(lags*p));
                        
                        newlogprob_marg_MCMC = logprob_marg_MCMC(:, :, chain);
                        newlogprob_joint_MCMC = logprob_joint_MCMC(:, :, chain);
                        
                        sumlogS = sum(log(S_MCMC(comp, :, chain)));
                        for j1 = 1:lags
                            diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                        end
                        diffstar2(:, :) = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                        
                        check2 = 1;
                        [newlogprob_marg_MCMC((lags+1):trainT, comp), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                        if ( check == 0 )
                            check2 = 0;
                        end
                        newlogprob_marg_MCMC((lags+1):trainT, comp) = newlogprob_marg_MCMC((lags+1):trainT, comp) - lags * sumlogS;
                        [newlogprob_joint_MCMC((lags+1):trainT, comp), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                        if ( check == 0 )
                            check2 = 0;
                        end
                        newlogprob_joint_MCMC((lags+1):trainT, comp) = newlogprob_joint_MCMC((lags+1):trainT, comp) - (lags + 1) * sumlogS;

                        if ( check2 == 1 )
                            z1 = prob_MCMC(1:it, chain) > 0;
                            maxstar = max(newlogprob_joint_MCMC((lags+1):trainT, z1==1), [], 2);
                            newloglike_MCMC = ones(1, trainT - lags) * (log(exp(newlogprob_joint_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                            maxstar = max(newlogprob_marg_MCMC((lags+1):trainT, z1==1), [], 2);
                            newloglike_MCMC = newloglike_MCMC - ones(1, trainT - lags) * (log(exp(newlogprob_marg_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                            
                            if ( newloglike_MCMC > 12000 )
                                stop;
                            end
                            
                            logaccept = heat(order(chain)) * (newloglike_MCMC - loglike_MCMC(chain));
                            logaccept = logaccept - 0.5 * newlambda_MCMC(j, k)^2 * tau_MCMC(k, comp, chain) * phi_MCMC(j, k, comp, chain);
                            logaccept = logaccept + 0.5 * lambda_MCMC(j, k, comp, chain)^2 * tau_MCMC(k, comp, chain) * phi_MCMC(j, k, comp, chain);
                            
                            accept = 1;
                            if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                                accept = 0;
                            elseif ( logaccept < 0 )
                                accept = exp(logaccept);
                            end
                        else
                            accept = 0;
                        end
                        
                        lambdaaccept(j, k, order(chain)) = lambdaaccept(j, k, order(chain)) + accept;
                        lambdacount(j, k, order(chain)) = lambdacount(j, k, order(chain)) + 1;
                        
                        if ( rand < accept )
                            lambda_MCMC(:, :, comp, chain) = newlambda_MCMC;
                            SuperP_MCMC(:, :, comp, chain) = newSuperP;
                            P_MCMC(:, :, comp, chain) = newP;
                            loglike_MCMC(chain) = newloglike_MCMC;
                            logprob_marg_MCMC(:, comp, chain) = newlogprob_marg_MCMC(:, comp);
                            logprob_joint_MCMC(:, comp, chain) = newlogprob_joint_MCMC(:, comp);
                        end
                        
                        loglambdasd(j, k, order(chain)) = loglambdasd(j, k, order(chain)) + 1 / it2^0.55 * (accept - 0.3);
                    end
                    
                else
                    newlambda_MCMC = lambda_MCMC(:, :, comp, chain);
                    
                    var1 = (lambdasum2(:, :, comp, order(chain)) - lambdasum1(:, comp, order(chain)) * lambdasum1(:, comp, order(chain))' / lambdasum3(comp, order(chain))) / (lambdasum3(comp, order(chain)) - 1);
                    newlambda_MCMC(:, k) = newlambda_MCMC(:, k) + exp(loglambdasd2(order(chain))) * chol(var1)' * randn(p, 1);
                    
                    newSuperP = SuperP_MCMC(:, :, comp, chain);
                    for i1 = 1:(lags + 1)
                        for j1 = 1:(lags + 1)
                            newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) - rho_MCMC(k, comp, chain)^(abs(i1 - j1)) * (lambda_MCMC(:, k, comp, chain) * lambda_MCMC(:, k, comp, chain)');
                            newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) + rho_MCMC(k, comp, chain)^(abs(i1 - j1)) * (newlambda_MCMC(:, k) * newlambda_MCMC(:, k)');
                        end
                    end
                    
                    newP = newSuperP(1:(lags*p), 1:(lags*p));
                    
                    newlogprob_marg_MCMC = logprob_marg_MCMC(:, :, chain);
                    newlogprob_joint_MCMC = logprob_joint_MCMC(:, :, chain);
                    
                    check2 = 1;
                    sumlogS = sum(log(S_MCMC(comp, :, chain)));
                    for j1 = 1:lags
                        diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                    end
                    diffstar2(:, :) = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                    
                    [newlogprob_marg_MCMC((lags+1):trainT, comp), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                    if ( check == 0 )
                        check2 = 0;
                    end
                    newlogprob_marg_MCMC((lags+1):trainT, comp) = newlogprob_marg_MCMC((lags+1):trainT, comp) - lags * sumlogS;
                    [newlogprob_joint_MCMC((lags+1):trainT, comp), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                    if ( check == 0 )
                        check2 = 0;
                    end
                    newlogprob_joint_MCMC((lags+1):trainT, comp) = newlogprob_joint_MCMC((lags+1):trainT, comp) - (lags + 1) * sumlogS;
                    
                    if ( check2 == 1 )
                        z1 = prob_MCMC(1:it, chain) > 0;
                        maxstar = max(newlogprob_joint_MCMC((lags+1):trainT, z1==1), [], 2);
                        newloglike_MCMC = ones(1, trainT - lags) * (log(exp(newlogprob_joint_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                        maxstar = max(newlogprob_marg_MCMC((lags+1):trainT, z1==1), [], 2);
                        newloglike_MCMC = newloglike_MCMC - ones(1, trainT - lags) * (log(exp(newlogprob_marg_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                        
                        
                        if ( newloglike_MCMC > 12000 )
                            stop;
                        end
                        
                        
                        logaccept = heat(order(chain)) * (newloglike_MCMC - loglike_MCMC(chain));
                        logaccept = logaccept - 0.5 * sum(newlambda_MCMC(:, k).^2 .* tau_MCMC(k, comp, chain) .* phi_MCMC(:, k, comp, chain));
                        logaccept = logaccept + 0.5 * sum(lambda_MCMC(:, k, comp, chain).^2 .* tau_MCMC(k, comp, chain) .* phi_MCMC(:, k, comp, chain));
                        
                        accept = 1;
                        if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                            accept = 0;
                        elseif ( logaccept < 0 )
                            accept = exp(logaccept);
                        end
                    else
                        accept = 0;
                    end
                    
                    lambdaaccept2(order(chain)) = lambdaaccept2(order(chain)) + accept;
                    lambdacount2(order(chain)) = lambdacount2(order(chain)) + 1;
                    
                    if ( rand < accept )
                        lambda_MCMC(:, :, comp, chain) = newlambda_MCMC;
                        SuperP_MCMC(:, :, comp, chain) = newSuperP;
                        P_MCMC(:, :, comp, chain) = newP;
                        loglike_MCMC(chain) = newloglike_MCMC;
                        logprob_marg_MCMC(:, comp, chain) = newlogprob_marg_MCMC(:, comp);
                        logprob_joint_MCMC(:, comp, chain) = newlogprob_joint_MCMC(:, comp);
                    end
                    
                    loglambdasd2(order(chain)) = loglambdasd2(order(chain)) + 1 / it2^0.55 * (accept - 0.3);
                end
            end
            
            lambdasum1(:, comp, order(chain)) = lambdasum1(:, comp, order(chain)) + lambda_MCMC(:, k, comp, chain);
            lambdasum2(:, :, comp, order(chain)) = lambdasum2(:, :, comp, order(chain)) + lambda_MCMC(:, k, comp, chain) * lambda_MCMC(:, k, comp, chain)';
            lambdasum3(comp, order(chain)) = lambdasum3(comp, order(chain)) + 1;
        end
        
        z1 = prob_MCMC(1:it, chain) > 0;
        maxstar = max(logprob_joint_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = ones(1, trainT - lags) * (log(exp(logprob_joint_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        maxstar = max(logprob_marg_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = checkloglike_MCMC - ones(1, trainT - lags) * (log(exp(logprob_marg_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        
        
        if ( abs(checkloglike_MCMC - loglike_MCMC(chain)) > 10^(-3) )
            stop;
        end
        
        for comp = 1:it
            
            if ( it2 < 20 )
                for i = 1:p
                    newphi_idio_MCMC = phi_idio_MCMC(i, comp, chain) * exp(exp(logphi_idiosd(i, order(chain))) * randn);
                    
                    newSuperP = SuperP_MCMC(:, :, comp, chain);
                    for i1 = 1:(lags + 1)
                        for j1 = 1:(lags + 1)
                            newSuperP((i1 - 1) * p + i, (j1 - 1) * p + i) = newSuperP((i1 - 1) * p + i, (j1 - 1) * p + i) - rho_idio_MCMC(i, comp, chain)^(abs(i1 - j1)) / phi_idio_MCMC(i, comp, chain);
                            newSuperP((i1 - 1) * p + i, (j1 - 1) * p + i) = newSuperP((i1 - 1) * p + i, (j1 - 1) * p + i) + rho_idio_MCMC(i, comp, chain)^(abs(i1 - j1)) / newphi_idio_MCMC;
                        end
                    end
                    
                    newP = newSuperP(1:(lags*p), 1:(lags*p));
                    
                    newlogprob_marg_MCMC = logprob_marg_MCMC(:, :, chain);
                    newlogprob_joint_MCMC = logprob_joint_MCMC(:, :, chain);
                    
                    sumlogS = sum(log(S_MCMC(comp, :, chain)));
                    for j1 = 1:lags
                        diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                    end
                    diffstar2(:, :) = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                    
                    check2 = 1;
                    [newlogprob_marg_MCMC((lags+1):trainT, comp), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                    if ( check == 0 )
                        check2 = 0;
                    end
                    newlogprob_marg_MCMC((lags+1):trainT, comp) = newlogprob_marg_MCMC((lags+1):trainT, comp) - lags * sumlogS;
                    [newlogprob_joint_MCMC((lags+1):trainT, comp), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                    if ( check == 0 )
                        check2 = 0;
                    end
                    newlogprob_joint_MCMC((lags+1):trainT, comp) = newlogprob_joint_MCMC((lags+1):trainT, comp) - (lags + 1) * sumlogS;
                    
                    if ( check2 == 1 )
                        z1 = prob_MCMC(1:it, chain) > 0;
                        maxstar = max(newlogprob_joint_MCMC((lags+1):trainT, z1==1), [], 2);
                        newloglike_MCMC = ones(1, trainT - lags) * (log(exp(newlogprob_joint_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                        maxstar = max(newlogprob_marg_MCMC((lags+1):trainT, z1==1), [], 2);
                        newloglike_MCMC = newloglike_MCMC - ones(1, trainT - lags) * (log(exp(newlogprob_marg_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                        
                        if ( newloglike_MCMC > 12000 )
                            stop;
                        end
                        
                        
                        logaccept = heat(order(chain)) * (newloglike_MCMC - loglike_MCMC(chain));
                        logaccept = logaccept + 0.5 * nu * (log(newphi_idio_MCMC) - log(phi_idio_MCMC(i, comp, chain))) - 0.5 * nu * (newphi_idio_MCMC - phi_idio_MCMC(i, comp, chain));
                        
                        accept = 1;
                        if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                            accept = 0;
                        elseif ( logaccept < 0 )
                            accept = exp(logaccept);
                        end
                    else
                        accept = 0;
                    end
                    
                    phi_idioaccept(i, order(chain)) = phi_idioaccept(i, order(chain)) + accept;
                    phi_idiocount(i, order(chain)) = phi_idiocount(i, order(chain)) + 1;
                    
                    if ( rand < accept )
                        phi_idio_MCMC(i, comp, chain) = newphi_idio_MCMC;
                        SuperP_MCMC(:, :, comp, chain) = newSuperP;
                        P_MCMC(:, :, comp, chain) = newP;
                        loglike_MCMC(chain) = newloglike_MCMC;
                        logprob_marg_MCMC(:, comp, chain) = newlogprob_marg_MCMC(:, comp);
                        logprob_joint_MCMC(:, comp, chain) = newlogprob_joint_MCMC(:, comp);
                    end
                    
                    logphi_idiosd(i, order(chain)) = logphi_idiosd(i, order(chain)) + 1 / it2^0.55 * (accept - 0.3);
                end
            else
                newtrans = log(phi_idio_MCMC(:, comp, chain));
                var1 = (phi_idiosum2(:, :, order(chain)) - phi_idiosum1(:, order(chain)) * phi_idiosum1(:, order(chain))' / phi_idiosum3(order(chain))) / (phi_idiosum3(order(chain)) - 1);
                newtrans = newtrans + exp(logphi_idiosd2(order(chain))) * chol(var1)' * randn(p, 1);
                newphi_idio_MCMC = exp(newtrans);
                
                newSuperP = SuperP_MCMC(:, :, comp, chain);
                for i1 = 1:(lags + 1)
                    for j1 = 1:(lags + 1)
                        for k = 1:p
                            newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) = newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) - rho_idio_MCMC(k, comp, chain)^(abs(i1 - j1)) / phi_idio_MCMC(k, comp, chain);
                            newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) = newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) + rho_idio_MCMC(k, comp, chain)^(abs(i1 - j1)) / newphi_idio_MCMC(k);
                        end
                    end
                end
                
                newP = newSuperP(1:(lags*p), 1:(lags*p));
                
                newlogprob_marg_MCMC = logprob_marg_MCMC(:, :, chain);
                newlogprob_joint_MCMC = logprob_joint_MCMC(:, :, chain);
                
                sumlogS = sum(log(S_MCMC(comp, :, chain)));
                for j1 = 1:lags
                    diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                end
                diffstar2(:, :) = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                
                check2 = 1;
                [newlogprob_marg_MCMC((lags+1):trainT, comp), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                if ( check == 0 )
                    check2 = 0;
                end
                newlogprob_marg_MCMC((lags+1):trainT, comp) = newlogprob_marg_MCMC((lags+1):trainT, comp) - lags * sumlogS;
                [newlogprob_joint_MCMC((lags+1):trainT, comp), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                if ( check == 0 )
                    check2 = 0;
                end
                newlogprob_joint_MCMC((lags+1):trainT, comp) = newlogprob_joint_MCMC((lags+1):trainT, comp) - (lags + 1) * sumlogS;
                
                if ( check2 == 1 )
                    z1 = prob_MCMC(1:it, chain) > 0;
                    maxstar = max(newlogprob_joint_MCMC((lags+1):trainT, z1==1), [], 2);
                    newloglike_MCMC = ones(1, trainT - lags) * (log(exp(newlogprob_joint_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                    maxstar = max(newlogprob_marg_MCMC((lags+1):trainT, z1==1), [], 2);
                    newloglike_MCMC = newloglike_MCMC - ones(1, trainT - lags) * (log(exp(newlogprob_marg_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                    
                    if ( newloglike_MCMC > 12000 )
                        stop;
                    end
                    
                    
                    logaccept = heat(order(chain)) * (newloglike_MCMC - loglike_MCMC(chain));
                    logaccept = logaccept + 0.5 * nu * (sum(log(newphi_idio_MCMC)) - sum(log(phi_idio_MCMC(:, comp, chain))));
                    logaccept = logaccept - 0.5 * nu * (sum(newphi_idio_MCMC) - sum(phi_idio_MCMC(:, comp, chain)));
                    
                    accept = 1;
                    if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                        accept = 0;
                    elseif ( logaccept < 0 )
                        accept = exp(logaccept);
                    end
                else
                    accept = 0;
                end
                
                phi_idioaccept2(order(chain)) = phi_idioaccept2(order(chain)) + accept;
                phi_idiocount2(order(chain)) = phi_idiocount2(order(chain)) + 1;
                
                if ( rand < accept )
                    phi_idio_MCMC(:, comp, chain) = newphi_idio_MCMC;
                    SuperP_MCMC(:, :, comp, chain) = newSuperP;
                    P_MCMC(:, :, comp, chain) = newP;
                    loglike_MCMC(chain) = newloglike_MCMC;
                    logprob_marg_MCMC(:, comp, chain) = newlogprob_marg_MCMC(:, comp);
                    logprob_joint_MCMC(:, comp, chain) = newlogprob_joint_MCMC(:, comp);
                end
                
                logphi_idiosd2(order(chain)) = logphi_idiosd2(order(chain)) + 1 / it2^0.55 * (accept - 0.3);
            end
            
            xtemp = log(phi_idio_MCMC(:, comp, chain));
            phi_idiosum1(:, order(chain)) = phi_idiosum1(:, order(chain)) + xtemp;
            phi_idiosum2(:, :, order(chain)) = phi_idiosum2(:, :, order(chain)) + xtemp * xtemp';
            phi_idiosum3(order(chain)) = phi_idiosum3(order(chain)) + 1;
            
        end
        
        z1 = prob_MCMC(1:it, chain) > 0;
        maxstar = max(logprob_joint_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = ones(1, trainT - lags) * (log(exp(logprob_joint_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        maxstar = max(logprob_marg_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = checkloglike_MCMC - ones(1, trainT - lags) * (log(exp(logprob_marg_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        
        
        if ( abs(checkloglike_MCMC - loglike_MCMC(chain)) > 10^(-3) )
            stop;
        end
        
        
        for comp = 1:it
            if ( it2 < 20 )
                
                for k = 1:Nfactors
                    
                    newtrans = log(rho_MCMC(k, comp, chain) - rhoepsilon) - log(1 - rhoepsilon - rho_MCMC(k, comp, chain));
                    newtrans = newtrans + exp(logrhosd(k, order(chain))) * randn;
                    newrho_MCMC = rhoepsilon + (1 - 2 * rhoepsilon) * exp(newtrans) / (exp(newtrans) + 1);
                    
                    newSuperP = SuperP_MCMC(:, :, comp, chain);
                    for i1 = 1:(lags + 1)
                        for j1 = 1:(lags + 1)
                            newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) - rho_MCMC(k, comp, chain)^(abs(i1 - j1)) * (lambda_MCMC(:, k, comp, chain) * lambda_MCMC(:, k, comp, chain)');
                            newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) + newrho_MCMC^(abs(i1 - j1)) * (lambda_MCMC(:, k, comp, chain) * lambda_MCMC(:, k, comp, chain)');
                        end
                    end
                    newP = newSuperP(1:(lags*p), 1:(lags*p));
                    
                    newlogprob_marg_MCMC = logprob_marg_MCMC(:, :, chain);
                    newlogprob_joint_MCMC = logprob_joint_MCMC(:, :, chain);
                    
                    sumlogS = sum(log(S_MCMC(comp, :, chain)));
                    for j1 = 1:lags
                        diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                    end
                    diffstar2(:, :) = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                    
                    check2 = 1;
                    [newlogprob_marg_MCMC((lags+1):trainT, comp), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                    if ( check == 0 )
                        check2 = 0;
                    end
                    newlogprob_marg_MCMC((lags+1):trainT, comp) = newlogprob_marg_MCMC((lags+1):trainT, comp) - lags * sumlogS;
                    [newlogprob_joint_MCMC((lags+1):trainT, comp), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                    if ( check == 0 )
                        check2 = 0;
                    end
                    newlogprob_joint_MCMC((lags+1):trainT, comp) = newlogprob_joint_MCMC((lags+1):trainT, comp) - (lags + 1) * sumlogS;
                    
                    if ( check2 == 1 )
                        z1 = prob_MCMC(1:it, chain) > 0;
                        maxstar = max(newlogprob_joint_MCMC((lags+1):trainT, z1==1), [], 2);
                        newloglike_MCMC = ones(1, trainT - lags) * (log(exp(newlogprob_joint_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                        maxstar = max(newlogprob_marg_MCMC((lags+1):trainT, z1==1), [], 2);
                        newloglike_MCMC = newloglike_MCMC - ones(1, trainT - lags) * (log(exp(newlogprob_marg_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                        
                        if ( newloglike_MCMC > 12000 )
                            stop;
                        end
                        
                        
                        logaccept = heat(order(chain)) * (newloglike_MCMC - loglike_MCMC(chain));
                        logaccept = logaccept - log(1 / (newrho_MCMC - rhoepsilon) + 1 / (1 - rhoepsilon - newrho_MCMC));
                        logaccept = logaccept + log(1 / (rho_MCMC(k, comp, chain) - rhoepsilon) + 1 / (1 - rhoepsilon - rho_MCMC(k, comp, chain)));
                        
                        accept = 1;
                        if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                            accept = 0;
                        elseif ( logaccept < 0 )
                            accept = exp(logaccept);
                        end
                    else
                        accept = 0;
                    end
                    
                    rhoaccept(k, order(chain)) = rhoaccept(k, order(chain)) + accept;
                    rhocount(k, order(chain)) = rhocount(k, order(chain)) + 1;
                    
                    if ( rand < accept )
                        rho_MCMC(k, comp, chain) = newrho_MCMC;
                        SuperP_MCMC(:, :, comp, chain) = newSuperP;
                        P_MCMC(:, :, comp, chain) = newP;
                        loglike_MCMC(chain) = newloglike_MCMC;
                        logprob_marg_MCMC(:, comp, chain) = newlogprob_marg_MCMC(:, comp);
                        logprob_joint_MCMC(:, comp, chain) = newlogprob_joint_MCMC(:, comp);
                    end
                    
                    logrhosd(k, order(chain)) = logrhosd(k, order(chain)) + 1 / it2^0.55 * (accept - 0.3);
                end
            else
                newtrans = log(rho_MCMC(:, comp, chain) - rhoepsilon) - log(1 - rhoepsilon - rho_MCMC(:, comp, chain));
                var1 = (rhosum2(:, :, order(chain)) - rhosum1(:, order(chain)) * rhosum1(:, order(chain))' / rhosum3(order(chain))) / (rhosum3(order(chain)) - 1);
                newtrans = newtrans + exp(logrhosd2(order(chain))) * chol(var1)' * randn(p, 1);
                newrho_MCMC = rhoepsilon + (1 - 2 * rhoepsilon) * exp(newtrans) ./ (exp(newtrans) + 1);
                
                newSuperP = SuperP_MCMC(:, :, comp, chain);
                for i1 = 1:(lags + 1)
                    for j1 = 1:(lags + 1)
                        for k = 1:p
                            newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) - rho_MCMC(k, comp, chain)^(abs(i1 - j1)) * (lambda_MCMC(:, k, comp, chain) * lambda_MCMC(:, k, comp, chain)');
                            newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) + newrho_MCMC(k)^(abs(i1 - j1)) * (lambda_MCMC(:, k, comp, chain) * lambda_MCMC(:, k, comp, chain)');
                        end
                    end
                end
                newP = newSuperP(1:(lags*p), 1:(lags*p));
                
                newlogprob_marg_MCMC = logprob_marg_MCMC(:, :, chain);
                newlogprob_joint_MCMC = logprob_joint_MCMC(:, :, chain);
                
                sumlogS = sum(log(S_MCMC(comp, :, chain)));
                for j1 = 1:lags
                    diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                end
                diffstar2(:, :) = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                
                check2 = 1;
                [newlogprob_marg_MCMC((lags+1):trainT, comp), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                if ( check == 0 )
                    check2 = 0;
                end
                newlogprob_marg_MCMC((lags+1):trainT, comp) = newlogprob_marg_MCMC((lags+1):trainT, comp) - lags * sumlogS;
                [newlogprob_joint_MCMC((lags+1):trainT, comp), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                if ( check == 0 )
                    check2 = 0;
                end
                newlogprob_joint_MCMC((lags+1):trainT, comp) = newlogprob_joint_MCMC((lags+1):trainT, comp) - (lags + 1) * sumlogS;
                
                if ( check2 == 1 )
                    z1 = prob_MCMC(1:it, chain) > 0;
                    maxstar = max(newlogprob_joint_MCMC((lags+1):trainT, z1==1), [], 2);
                    newloglike_MCMC = ones(1, trainT - lags) * (log(exp(newlogprob_joint_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                    maxstar = max(newlogprob_marg_MCMC((lags+1):trainT, z1==1), [], 2);
                    newloglike_MCMC = newloglike_MCMC - ones(1, trainT - lags) * (log(exp(newlogprob_marg_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                    
                    if ( newloglike_MCMC > 12000 )
                        stop;
                    end
                    
                    
                    logaccept = heat(order(chain)) * (newloglike_MCMC - loglike_MCMC(chain));
                    logaccept = logaccept - sum(log(1 ./ (newrho_MCMC - rhoepsilon) + 1 ./ (1 - rhoepsilon - newrho_MCMC)));
                    logaccept = logaccept + sum(log(1 ./ (rho_MCMC(:, comp, chain) - rhoepsilon) + 1 ./ (1 - rhoepsilon - rho_MCMC(:, comp, chain))));
                    
                    accept = 1;
                    if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                        accept = 0;
                    elseif ( logaccept < 0 )
                        accept = exp(logaccept);
                    end
                else
                    accept = 0;
                end
                
                rhoaccept2(order(chain)) = rhoaccept2(order(chain)) + accept;
                rhocount2(order(chain)) = rhocount2(order(chain)) + 1;
                
                if ( rand < accept )
                    rho_MCMC(:, comp, chain) = newrho_MCMC;
                    SuperP_MCMC(:, :, comp, chain) = newSuperP;
                    P_MCMC(:, :, comp, chain) = newP;
                    loglike_MCMC(chain) = newloglike_MCMC;
                    logprob_marg_MCMC(:, comp, chain) = newlogprob_marg_MCMC(:, comp);
                    logprob_joint_MCMC(:, comp, chain) = newlogprob_joint_MCMC(:, comp);
                end
                
                logrhosd2(order(chain)) = logrhosd2(order(chain)) + 1 / it2^0.55 * (accept - 0.3);
            end
            
            xtemp = log(rho_MCMC(:, comp, chain) - rhoepsilon) - log(1 - rhoepsilon - rho_MCMC(:, comp, chain));
            rhosum1(:, order(chain)) = rhosum1(:, order(chain)) + xtemp;
            rhosum2(:, :, order(chain)) = rhosum2(:, :, order(chain)) + xtemp * xtemp';
            rhosum3(order(chain)) = rhosum3(order(chain)) + 1;
        end
        
        z1 = prob_MCMC(1:it, chain) > 0;
        maxstar = max(logprob_joint_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = ones(1, trainT - lags) * (log(exp(logprob_joint_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        maxstar = max(logprob_marg_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = checkloglike_MCMC - ones(1, trainT - lags) * (log(exp(logprob_marg_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        
        if ( abs(checkloglike_MCMC - loglike_MCMC(chain)) > 10^(-3) )
            stop;
        end
        
        for comp = 1:it
            
            if ( it2 < 20 )
                for i = 1:p
                    
                    newtrans = log(rho_idio_MCMC(i, comp, chain) - rhoepsilon) - log(1 - rhoepsilon - rho_idio_MCMC(i, comp, chain));
                    newtrans = newtrans + exp(logrho_idiosd(i, order(chain))) * randn;
                    newrho_idio_MCMC = rhoepsilon + (1 - 2 * rhoepsilon) * exp(newtrans) / (exp(newtrans) + 1);
                    
                    newSuperP = SuperP_MCMC(:, :, comp, chain);
                    for i1 = 1:(lags + 1)
                        for j1 = 1:(lags + 1)
                            newSuperP((i1 - 1) * p + i, (j1 - 1) * p + i) = newSuperP((i1 - 1) * p + i, (j1 - 1) * p + i) - rho_idio_MCMC(i, comp, chain)^(abs(i1 - j1)) / phi_idio_MCMC(i, comp, chain);
                            newSuperP((i1 - 1) * p + i, (j1 - 1) * p + i) = newSuperP((i1 - 1) * p + i, (j1 - 1) * p + i) + newrho_idio_MCMC^(abs(i1 - j1)) / phi_idio_MCMC(i, comp, chain);
                        end
                    end
                    
                    newP = newSuperP(1:(lags*p), 1:(lags*p));
                    
                    newlogprob_marg_MCMC = logprob_marg_MCMC(:, :, chain);
                    newlogprob_joint_MCMC = logprob_joint_MCMC(:, :, chain);
                    
                    sumlogS = sum(log(S_MCMC(comp, :, chain)));
                    for j1 = 1:lags
                        diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                    end
                    diffstar2(:, :) = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                    
                    check2 = 1;
                    [newlogprob_marg_MCMC((lags+1):trainT, comp), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                    if ( check == 0 )
                        check2 = 0;
                    end
                    newlogprob_marg_MCMC((lags+1):trainT, comp) = newlogprob_marg_MCMC((lags+1):trainT, comp) - lags * sumlogS;
                    [newlogprob_joint_MCMC((lags+1):trainT, comp), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                    if ( check == 0 )
                        check2 = 0;
                    end
                    newlogprob_joint_MCMC((lags+1):trainT, comp) = newlogprob_joint_MCMC((lags+1):trainT, comp) - (lags + 1) * sumlogS;

                    if ( check2 == 1 )
                        z1 = prob_MCMC(1:it, chain) > 0;
                        maxstar = max(newlogprob_joint_MCMC((lags+1):trainT, z1==1), [], 2);
                        newloglike_MCMC = ones(1, trainT - lags) * (log(exp(newlogprob_joint_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                        maxstar = max(newlogprob_marg_MCMC((lags+1):trainT, z1==1), [], 2);
                        newloglike_MCMC = newloglike_MCMC - ones(1, trainT - lags) * (log(exp(newlogprob_marg_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                        
                        if ( newloglike_MCMC > 12000 )
                            stop;
                        end
                        
                        
                        logaccept = heat(order(chain)) * (newloglike_MCMC - loglike_MCMC(chain));
                        logaccept = logaccept - log(1 / (newrho_idio_MCMC - rhoepsilon) + 1 / (1 - rhoepsilon - newrho_idio_MCMC));
                        logaccept = logaccept + log(1 / (rho_idio_MCMC(i, comp, chain) - rhoepsilon) + 1 / (1 - rhoepsilon - rho_idio_MCMC(i, comp, chain)));
                        
                        accept = 1;
                        if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                            accept = 0;
                        elseif ( logaccept < 0 )
                            accept = exp(logaccept);
                        end
                    else
                        accept = 0;
                    end
                    
                    rho_idioaccept(i, order(chain)) = rho_idioaccept(i, order(chain)) + accept;
                    rho_idiocount(i, order(chain)) = rho_idiocount(i, order(chain)) + 1;
                    
                    if ( rand < accept )
                        rho_idio_MCMC(i, comp, chain) = newrho_idio_MCMC;
                        SuperP_MCMC(:, :, comp, chain) = newSuperP;
                        P_MCMC(:, :, comp, chain) = newP;
                        loglike_MCMC(chain) = newloglike_MCMC;
                        logprob_marg_MCMC(:, comp, chain) = newlogprob_marg_MCMC(:, comp);
                        logprob_joint_MCMC(:, comp, chain) = newlogprob_joint_MCMC(:, comp);
                    end
                    
                    logrho_idiosd(i, order(chain)) = logrho_idiosd(i, order(chain)) + 1 / it2^0.55 * (accept - 0.3);
                end
            else
                newtrans = log(rho_idio_MCMC(:, comp, chain) - rhoepsilon) - log(1 - rhoepsilon - rho_idio_MCMC(:, comp, chain));
                var1 = (rho_idiosum2(:, :, order(chain)) - rho_idiosum1(:, order(chain)) * rho_idiosum1(:, order(chain))' / rho_idiosum3(order(chain))) / (rho_idiosum3(order(chain)) - 1);
                newtrans = newtrans + exp(logrho_idiosd2(order(chain))) * chol(var1)' * randn(p, 1);
                newrho_idio_MCMC = rhoepsilon + (1 - 2 * rhoepsilon) * exp(newtrans) ./ (exp(newtrans) + 1);
                
                newSuperP = SuperP_MCMC(:, :, comp, chain);
                for i1 = 1:(lags + 1)
                    for j1 = 1:(lags + 1)
                        for k = 1:p
                            newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) = newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) - rho_idio_MCMC(k, comp, chain)^(abs(i1 - j1)) / phi_idio_MCMC(k, comp, chain);
                            newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) = newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) + newrho_idio_MCMC(k)^(abs(i1 - j1)) / phi_idio_MCMC(k, comp, chain);
                        end
                    end
                end
                
                newP = newSuperP(1:(lags*p), 1:(lags*p));
                
                newlogprob_marg_MCMC = logprob_marg_MCMC(:, :, chain);
                newlogprob_joint_MCMC = logprob_joint_MCMC(:, :, chain);
                
                sumlogS = sum(log(S_MCMC(comp, :, chain)));
                for j1 = 1:lags
                    diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                end
                diffstar2(:, :) = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * S_MCMC(comp, :, chain));
                
                check2 = 1;
                [newlogprob_marg_MCMC((lags+1):trainT, comp), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                if ( check == 0 )
                    check2 = 0;
                end
                newlogprob_marg_MCMC((lags+1):trainT, comp) = newlogprob_marg_MCMC((lags+1):trainT, comp) - lags * sumlogS;
                [newlogprob_joint_MCMC((lags+1):trainT, comp), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                if ( check == 0 )
                    check2 = 0;
                end
                newlogprob_joint_MCMC((lags+1):trainT, comp) = newlogprob_joint_MCMC((lags+1):trainT, comp) - (lags + 1) * sumlogS;

                if ( check2 == 1 )
                    z1 = prob_MCMC(1:it, chain) > 0;
                    maxstar = max(newlogprob_joint_MCMC((lags+1):trainT, z1==1), [], 2);
                    newloglike_MCMC = ones(1, trainT - lags) * (log(exp(newlogprob_joint_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                    maxstar = max(newlogprob_marg_MCMC((lags+1):trainT, z1==1), [], 2);
                    newloglike_MCMC = newloglike_MCMC - ones(1, trainT - lags) * (log(exp(newlogprob_marg_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                    
                    if ( newloglike_MCMC > 12000 )
                        stop;
                    end
                    
                    
                    logaccept = heat(order(chain)) * (newloglike_MCMC - loglike_MCMC(chain));
                    logaccept = logaccept - sum(log(1 ./ (newrho_idio_MCMC - rhoepsilon) + 1 ./ (1 - rhoepsilon - newrho_idio_MCMC)));
                    logaccept = logaccept + sum(log(1 ./ (rho_idio_MCMC(:, comp, chain) - rhoepsilon) + 1 ./ (1 - rhoepsilon - rho_idio_MCMC(:, comp, chain))));
                    
                    accept = 1;
                    if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                        accept = 0;
                    elseif ( logaccept < 0 )
                        accept = exp(logaccept);
                    end
                else
                    accept = 0;
                end
                
                rho_idioaccept2(order(chain)) = rho_idioaccept2(order(chain)) + accept;
                rho_idiocount2(order(chain)) = rho_idiocount2(order(chain)) + 1;
                
                if ( rand < accept )
                    rho_idio_MCMC(:, comp, chain) = newrho_idio_MCMC;
                    SuperP_MCMC(:, :, comp, chain) = newSuperP;
                    P_MCMC(:, :, comp, chain) = newP;
                    loglike_MCMC(chain) = newloglike_MCMC;
                    logprob_marg_MCMC(:, comp, chain) = newlogprob_marg_MCMC(:, comp);
                    logprob_joint_MCMC(:, comp, chain) = newlogprob_joint_MCMC(:, comp);
                end
                
                logrho_idiosd2(order(chain)) = logrho_idiosd2(order(chain)) + 1 / it2^0.55 * (accept - 0.3);
            end
            
            xtemp = log(rho_idio_MCMC(:, comp, chain) - rhoepsilon) - log(1 - rhoepsilon - rho_idio_MCMC(:, comp, chain));
            rho_idiosum1(:, order(chain)) = rho_idiosum1(:, order(chain)) + xtemp;
            rho_idiosum2(:, :, order(chain)) = rho_idiosum2(:, :, order(chain)) + xtemp * xtemp';
            rho_idiosum3(order(chain)) = rho_idiosum3(order(chain)) + 1;
            
        end
        
        
        z1 = prob_MCMC(1:it, chain) > 0;
        maxstar = max(logprob_joint_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = ones(1, trainT - lags) * (log(exp(logprob_joint_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        maxstar = max(logprob_marg_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = checkloglike_MCMC - ones(1, trainT - lags) * (log(exp(logprob_marg_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        
        
        if ( abs(checkloglike_MCMC - loglike_MCMC(chain)) > 10^(-3) )
            stop;
        end
        
        
        for comp = 1:it
            % Update phi
            for k = 1:Nfactors
                phi_MCMC(:, k, comp, chain) = gamrnd((nu + 1) / 2, 2 ./ (nu + tau_MCMC(k, comp, chain) * lambda_MCMC(:, k, comp, chain).^2));
            end
            
            % Update delta
            sum1 = zeros(Nfactors, 1);
            for k = 1:Nfactors
                sum1(k) = sum(phi_MCMC(:, k, comp, chain) .* lambda_MCMC(:, k, comp, chain).^2);
            end
            
            olddelta_MCMC = delta_MCMC(1, comp, chain);
            delta_MCMC(1, comp, chain) = gamrnd(a1 + p * Nfactors / 2, 1 / (1 + 0.5 * sum(tau_MCMC(1:Nfactors, comp, chain) / delta_MCMC(1, comp, chain) .* sum1)));
            tau_MCMC(1, comp, chain) = delta_MCMC(1, comp, chain);
            tau_MCMC(2:Nfactors, comp, chain) = tau_MCMC(2:Nfactors, comp, chain) * delta_MCMC(1, comp, chain) / olddelta_MCMC;
            for k = 2:Nfactors
                olddelta_MCMC = delta_MCMC(k, comp, chain);
                delta_MCMC(k, comp, chain) = gamrnd(a2 + p * (Nfactors - k + 1) / 2, 1 / (1 + 0.5 * sum(tau_MCMC(k:Nfactors, comp, chain) / delta_MCMC(k, comp, chain) .* sum1(k:Nfactors))));
                tau_MCMC(k, comp, chain) = tau_MCMC(k - 1, comp, chain) * delta_MCMC(k, comp, chain);
                tau_MCMC((k+1):Nfactors, comp, chain) = tau_MCMC((k+1):Nfactors, comp, chain) * delta_MCMC(k, comp, chain) / olddelta_MCMC;
            end
        end
        
            
        % Update S
        for comp = 1:it
            for i = 1:p
                newS_MCMC = S_MCMC(comp, :, chain);
                newS_MCMC(i) = newS_MCMC(i) * exp(exp(logSsd(i, order(chain))) * randn);
                
                sumlogS = sum(log(newS_MCMC));
                
                newlogprob_marg_MCMC = logprob_marg_MCMC(:, :, chain);
                newlogprob_joint_MCMC = logprob_joint_MCMC(:, :, chain);
                
                for j1 = 1:lags
                    diffstar1(:, ((j1-1)*p+1):(j1*p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * newS_MCMC);
                end
                diffstar2 = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu_MCMC(comp, :, chain)) ./ (ones(trainT - lags, 1) * newS_MCMC);
                
                check2 = 1;
                [newlogprob_marg_MCMC((lags+1):trainT, comp), check] = logmvnpdf(diffstar1, zeros(1, lags * p), P_MCMC(:, :, comp, chain));
                if ( check == 0 )
                    check2 = 0;
                end
                newlogprob_marg_MCMC((lags+1):trainT, comp) = newlogprob_marg_MCMC((lags+1):trainT, comp) - lags * sumlogS;
                [newlogprob_joint_MCMC((lags+1):trainT, comp), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), SuperP_MCMC(:, :, comp, chain));
                if ( check == 0 )
                    check2 = 0;
                end
                newlogprob_joint_MCMC((lags+1):trainT, comp) = newlogprob_joint_MCMC((lags+1):trainT, comp) - (lags + 1) * sumlogS;

                if ( check2 == 1 )
                    z1 = prob_MCMC(1:it, chain) > 0;
                    maxstar = max(newlogprob_joint_MCMC((lags+1):trainT, z1==1), [], 2);
                    newloglike_MCMC = ones(1, trainT - lags) * (log(exp(newlogprob_joint_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                    maxstar = max(newlogprob_marg_MCMC((lags+1):trainT, z1==1), [], 2);
                    newloglike_MCMC = newloglike_MCMC - ones(1, trainT - lags) * (log(exp(newlogprob_marg_MCMC((lags+1):trainT, z1==1) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
                    
                    if ( newloglike_MCMC > 12000 )
                        stop;
                    end
                    
                    
                    logaccept = heat(order(chain)) * (newloglike_MCMC - loglike_MCMC(chain));
                    logaccept = logaccept - (Salpha + 1) * log(newS_MCMC(i)) - (Salpha - 1) * Smu_MCMC(i, chain) / newS_MCMC(i);
                    logaccept = logaccept + (Salpha + 1) * log(S_MCMC(comp, i, chain)) + (Salpha - 1) * Smu_MCMC(i, chain) / S_MCMC(comp, i, chain);
                    logaccept = logaccept + log(newS_MCMC(i)) - log(S_MCMC(comp, i, chain));
                    
                    accept = 1;
                    if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                        accept = 0;
                    elseif ( logaccept < 0 )
                        accept = exp(logaccept);
                    end
                else
                    accept = 0;
                end
                
                Saccept(i, order(chain)) = Saccept(i, order(chain)) + accept;
                Scount(i, order(chain)) = Scount(i, order(chain)) + 1;
                
                if ( rand < accept )
                    S_MCMC(comp, i, chain) = newS_MCMC(i);
                    loglike_MCMC(chain) = newloglike_MCMC;
                    logprob_marg_MCMC(:, comp, chain) = newlogprob_marg_MCMC(:, comp);
                    logprob_joint_MCMC(:, comp, chain) = newlogprob_joint_MCMC(:, comp);
                end
                
                logSsd(i, order(chain)) = logSsd(i, order(chain)) + 1 / it2^0.55 * (accept - 0.3);
            end
        end
        
        z1 = prob_MCMC(1:it, chain) > 0;
        maxstar = max(logprob_joint_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = ones(1, trainT - lags) * (log(exp(logprob_joint_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        maxstar = max(logprob_marg_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = checkloglike_MCMC - ones(1, trainT - lags) * (log(exp(logprob_marg_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        
        
        if ( abs(checkloglike_MCMC - loglike_MCMC(chain)) > 10^(-3) )
            stop;
        end
        
        % Update Smu
        for i = 1:p
            Smu_MCMC(i, chain) = gamrnd(it * Salpha + Smu_alpha, 1 / ((Salpha - 1) * sum(1./S_MCMC(1:it, i, chain)) + Smu_beta));
        end
        
        % Update V
        for comp = 1:it
            newV_MCMC = V_MCMC(:, chain);
            trans = log(V_MCMC(comp, chain) - Vepsilon) - log(1 - Vepsilon - V_MCMC(comp, chain));
            newtrans = trans + exp(logVsd(order(chain))) * randn;
            newV_MCMC(comp) = Vepsilon + (1 - 2 * Vepsilon) * exp(newtrans) / (1 + exp(newtrans));
            
            newprob_MCMC = prob_MCMC(:, chain);
            prodV = 1;
            for i = 1:it
                newprob_MCMC(i) = newV_MCMC(i) * prodV;
                prodV = prodV * (1 - newV_MCMC(i));
            end
            newprob_MCMC = newprob_MCMC / sum(newprob_MCMC);
            
            z1 = newprob_MCMC(1:it) > 0;
            maxstar = max(permute(logprob_joint_MCMC((lags+1):trainT, z1==1, chain), [2 1 3]))';
            newloglike_MCMC = ones(1, trainT - lags) * (log(exp(logprob_joint_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * newprob_MCMC(z1==1)) + maxstar);
            maxstar = max(permute(logprob_marg_MCMC((lags+1):trainT, z1==1, chain), [2 1 3]))';
            newloglike_MCMC = newloglike_MCMC - ones(1, trainT - lags) * (log(exp(logprob_marg_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * newprob_MCMC(z1==1)) + maxstar);
            
            if ( newloglike_MCMC > 12000 )
                stop;
            end
            
            
            logaccept = heat(order(chain)) * (newloglike_MCMC - loglike_MCMC(chain));
            logaccept = logaccept + (M_MCMC(chain) - 1) * (log(1 - newV_MCMC(comp)) - log(1 - V_MCMC(comp, chain)));
            logaccept = logaccept - log(1 / (newV_MCMC(comp) - Vepsilon) + 1 / (1 - Vepsilon - newV_MCMC(comp))) + log(1 / (V_MCMC(comp, chain) - Vepsilon) + 1 / (1 - Vepsilon - V_MCMC(comp, chain)));
            
            accept = 1;
            if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                accept = 0;
            elseif ( logaccept < 0 )
                accept = exp(logaccept);
            end
            
            Vaccept(order(chain)) = Vaccept(order(chain)) + accept;
            Vcount(order(chain)) = Vcount(order(chain)) + 1;
            
            if ( rand < accept )
                V_MCMC(comp, chain) = newV_MCMC(comp);
                prob_MCMC(:, chain) = newprob_MCMC;
                loglike_MCMC(chain) = newloglike_MCMC;
            end
            
            logVsd(order(chain)) = logVsd(order(chain)) + 1 / it2^0.55 * (accept - 0.3);
        end
        
        
        z1 = prob_MCMC(1:it, chain) > 0;
        maxstar = max(logprob_joint_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = ones(1, trainT - lags) * (log(exp(logprob_joint_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        maxstar = max(logprob_marg_MCMC((lags+1):trainT, z1==1, chain), [], 2);
        checkloglike_MCMC = checkloglike_MCMC - ones(1, trainT - lags) * (log(exp(logprob_marg_MCMC((lags+1):trainT, z1==1, chain) - maxstar * ones(1, sum(z1))) * prob_MCMC(z1==1, chain)) + maxstar);
        
        
        if ( abs(checkloglike_MCMC - loglike_MCMC(chain)) > 10^(-3) )
            stop;
        end
        
        M_MCMC(chain) = gamrnd(1 + it, 1 / (1 - sum(log(1 - V_MCMC(1:it, chain)))));
        
    end
    
    %  'Swap'
    if ( numbofchains > 1 )
        for k = 1:5
            
            % Swap move
            check = 1;
            while ( check == 1 )
                chain1 = ceil(rand * numbofchains);
                check = (order(chain1) == numbofchains);
            end
            chain2 = find(order == order(chain1) + 1);
            
            logaccept = (heat(order(chain1)) - heat(order(chain2))) * (loglike_MCMC(chain2) - loglike_MCMC(chain1));
            
            accept = 1;
            if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                accept = 0;
            elseif ( logaccept < 0 )
                accept = exp(logaccept);
            end
            totalaccept2(order(chain1)) = totalaccept2(order(chain1)) + accept;
            totalcount2(order(chain1)) = totalcount2(order(chain1)) + 1;
            
            rhostar_PT(order(chain1)) = rhostar_PT(order(chain1)) + 1 / it^0.55 * (accept - 0.234);
            if ( rhostar_PT(order(chain1)) > 4 )
                rhostar_PT(order(chain1)) = 4;
            end
            
            for j = 1:(length(heat)-1)
                heat(j + 1) = heat(j) * exp(- exp(rhostar_PT(j)));
            end
            
            if ( rand < accept )
                order(chain1) = order(chain1) + 1;
                order(chain2) = order(chain2) - 1;
            end
        end
    end
    
    
    d.Value = (run * numbofits + it2)/(tot_runs * numbofits); 
    d.Message = 'Current Progress';
    
    
    if ( mod(it2, 10) == 0 )
        top_chain = find(order == 1);
        

        
%         minstar = zeros(1, numbofchains);
%         maxstar = zeros(1, numbofchains);
%         disp(['it2 = ' num2str(it2)]);
%         disp(['M = ' num2str(M_MCMC(top_chain))]);
%         disp(['Smu = ' num2str(Smu_MCMC(:, top_chain)')]);
%         for k = 1:numbofchains
%             minstar(k) = min(muaccept(:, k)./mucount(:, k));
%             maxstar(k) = max(muaccept(:, k)./mucount(:, k));
%         end
%         disp('mu accept = ')
%         disp(num2str(minstar));
%         disp(num2str(maxstar));
%         if ( it2 < 20 )
%             for k = 1:numbofchains
%                 minstar(k) = min(rhoaccept(:, k)./rhocount(:, k));
%                 maxstar(k) = max(rhoaccept(:, k)./rhocount(:, k));
%             end
%             disp('rho accept = ');
%             disp(num2str(minstar));
%             disp(num2str(maxstar));
%             for k = 1:numbofchains
%                 minstar(k) = min(rho_idioaccept(:, k)./rho_idiocount(:, k));
%                 maxstar(k) = max(rho_idioaccept(:, k)./rho_idiocount(:, k));
%             end
%             disp('rho_idio accept = ');
%             disp(num2str(minstar));
%             disp(num2str(maxstar));
%             for k = 1:numbofchains
%                 minstar(k) = min(phi_idioaccept(:, k)./phi_idiocount(:, k));
%                 maxstar(k) = max(phi_idioaccept(:, k)./phi_idiocount(:, k));
%             end
%             disp('phi_idio accept = ');
%             disp(num2str(minstar));
%             disp(num2str(maxstar));
%             for k = 1:numbofchains
%                 minstar(k) = min(min(lambdaaccept(:, :, k)./lambdacount(:, :, k)));
%                 maxstar(k) = max(max(lambdaaccept(:, :, k)./lambdacount(:, :, k)));
%             end
%             disp('lambda accept = ');
%             disp(num2str(minstar));
%             disp(num2str(maxstar));
%         else
%             for k = 1:numbofchains
%                 minstar(k) = min(rhoaccept2(k)./rhocount2(k));
%                 maxstar(k) = max(rhoaccept2(k)./rhocount2(k));
%             end
%             disp('rho2 accept = ');
%             disp(num2str(minstar));
%             disp(num2str(maxstar));
%             for k = 1:numbofchains
%                 minstar(k) = min(rho_idioaccept2(k)/rho_idiocount2(k));
%                 maxstar(k) = max(rho_idioaccept2(k)/rho_idiocount2(k));
%             end
%             disp('rho_idio accept2 = ');
%             disp(num2str(minstar));
%             disp(num2str(maxstar));
%             for k = 1:numbofchains
%                 minstar(k) = min(phi_idioaccept2(k)/phi_idiocount2(k));
%                 maxstar(k) = max(phi_idioaccept2(k)/phi_idiocount2(k));
%             end
%             disp('phi_idio accept2 = ');
%             disp(num2str(minstar));
%             disp(num2str(maxstar));
%             for k = 1:numbofchains
%                 minstar(k) = min(min(lambdaaccept2(k)./lambdacount2(k)));
%                 maxstar(k) = max(max(lambdaaccept2(k)./lambdacount2(k)));
%             end
%             disp('lambda2 accept = ');
%             disp(num2str(minstar));
%             disp(num2str(maxstar));
%         end
%         for k = 1:numbofchains
%             minstar(k) = min(Saccept(:, k)./Scount(:, k));
%             maxstar(k) = max(Saccept(:, k)./Scount(:, k));
%         end
%         disp('S accept = ')
%         disp(num2str(minstar));
%         disp(num2str(maxstar));
%         disp(['V accept = ' num2str(Vaccept./Vcount)]);
%         disp(['M accept = ' num2str(Maccept./Mcount)]);
%         hold1 = zeros(1, length(order));
%         for i = 1:length(order)
%             hold1(order(i)) = loglike_MCMC(i);
%         end
%         disp(['loglike = ' num2str(hold1)]);
%         disp(['swap accept = ' num2str(totalaccept2./totalcount2)]);
%         disp(['heat = ' num2str(heat)]);
%         disp(' ');
    end
    
    if ( (it2 > burnin) && ( mod(it2 - burnin, every) == 0) )
        top_chain = find(order == 1);
        
        mu(:, (it2 - burnin) / every, 1:it) = mu_MCMC(:, :, top_chain)';
        P(:, :, (it2 - burnin) / every, 1:it) = P_MCMC(:, :, :, top_chain);
        SuperP(:, :, (it2 - burnin) / every, 1:it) = SuperP_MCMC(:, :, :, top_chain);
        rho(:, (it2 - burnin) / every, 1:it) = rho_MCMC(:, :, top_chain);
        rho_idio(:, (it2 - burnin) / every, 1:it) = rho_idio_MCMC(:, :, top_chain);
        lambda(:, :, (it2 - burnin) / every, 1:it) = lambda_MCMC(:, :, :, top_chain);
        phi_idio(:, (it2 - burnin) / every, 1:it) = phi_idio_MCMC(:, :, top_chain);
        phi(:, :, (it2 - burnin) / every, 1:it) = phi_MCMC(:, :, :, top_chain);
        tau(:, (it2 - burnin) / every, 1:it) = tau_MCMC(:, :, top_chain);
        delta(:, (it2 - burnin) / every, 1:it) = delta_MCMC(:, :, top_chain);
        S(:, (it2 - burnin) / every, 1:it) = S_MCMC(:, :, top_chain)';
        prob((it2 - burnin) / every, 1:it) = prob_MCMC(:, top_chain);
        V((it2 - burnin) / every, 1:it) = V_MCMC(:, top_chain);
        loglike((it2 - burnin) / every) = loglike_MCMC(top_chain);
        M((it2 - burnin) / every) = M_MCMC(top_chain);
        logprob_marg(:, 1:it, (it2 - burnin) / every) = logprob_marg_MCMC(:, :, top_chain);
        logprob_joint(:, 1:it, (it2 - burnin) / every) = logprob_joint_MCMC(:, :, top_chain);
        Smu(:, (it2 - burnin) / every) = Smu_MCMC(:, top_chain);
    end
end

logmusd = logmusd(:, 1);
loglambdasd2 = loglambdasd2(1);
logphi_idiosd2 = logphi_idiosd2(1);
logrhosd2 = logrhosd2(1);
logrho_idiosd2 = logrho_idiosd2(1);
logSsd = logSsd(:, 1);
logVsd = logVsd(:, 1);
lambdasum1 = lambdasum1(:, 1);
lambdasum2 = lambdasum2(:, :, 1);
lambdasum3 = lambdasum3(1);
phi_idiosum1 = phi_idiosum1(:, 1);
phi_idiosum2 = phi_idiosum2(:, :, 1);
phi_idiosum3 = phi_idiosum3(1);
rhosum1 = rhosum1(:, 1);
rhosum2 = rhosum2(:, :, 1);
rhosum3 = rhosum3(1);
rho_idiosum1 = rho_idiosum1(:, 1);
rho_idiosum2 = rho_idiosum2(:, :, 1);
rho_idiosum3 = rho_idiosum3(1);


muaccept = muaccept(:, 1);
mucount = mucount(:, 1);
rhoaccept2 = rhoaccept(:, 1);
rhocount2 = rhocount(:, 1);
rho_idioaccept2 = rho_idioaccept(:, 1);
rho_idiocount2 = rho_idiocount(:, 1);
lambdaaccept2 = lambdaaccept2(1);
lambdacount2 = lambdacount2(1);
phi_idioaccept2 = phi_idioaccept(:, 1);
phi_idiocount2 = phi_idiocount(:, 1);
Saccept = Saccept(:, 1);
Scount = Scount(:, 1);
Vaccept = Vaccept(1);
Vcount = Vcount(1);

storeESS = zeros(1, top_trunc - start_trunc + 1);


partstar = zeros(1, numbofparts);
rat = zeros(1, top_trunc - start_trunc + 1);

logweight = zeros(1, numbofparts);
check_trunc = 0;
it = start_trunc - 1;

while ( check_trunc == 0 )
    
    it = it + 1;
    
    if ( it > size(V, 2) )
        V = [V ones(numbofparts, top_trunc)];
        prob = [prob zeros(numbofparts, top_trunc)];
        mutemp = zeros(p, numbofparts, top_trunc+size(mu, 3));
        mutemp(:, :, 1:size(mu, 3)) = mu;
        mu = mutemp;
        Stemp = zeros(p, numbofparts, top_trunc+size(S, 3));
        Stemp(:, :, 1:size(S, 3)) = S;
        S = Stemp;
        phi_idiotemp = zeros(p, numbofparts, top_trunc+size(phi_idio, 3));
        phi_idiotemp(:, :, 1:size(phi_idio, 3)) = phi_idio;
        phi_idio = phi_idiotemp;
        Ptemp = zeros(K - p, K - p, numbofparts, top_trunc+size(P, 4));
        Ptemp(:, :, :, 1:size(P, 4)) = P;
        P = Ptemp;
        SuperPtemp = zeros(K, K, numbofparts, top_trunc+size(SuperP, 4));
        SuperPtemp(:, :, :, 1:size(SuperP, 4)) = SuperP;
        SuperP = SuperPtemp;
        lambdatemp = zeros(p, Nfactors, numbofparts, top_trunc+size(lambda, 4));
        lambdatemp(:, :, :, 1:size(lambda, 4)) = lambda;
        lambda = lambdatemp;
        phitemp = zeros(p, Nfactors, numbofparts, top_trunc+size(phi, 4));
        phitemp(:, :, :, 1:size(phi, 4)) = phi;
        phi = phitemp;
        deltatemp = zeros(Nfactors, numbofparts, top_trunc+size(delta, 3));
        deltatemp(:, :, 1:size(delta, 3)) = delta;
        delta = deltatemp;
        tautemp = zeros(Nfactors, numbofparts, top_trunc+size(tau, 3));
        tautemp(:, :, 1:size(tau, 3)) = tau;
        tau = tautemp;
        rhotemp = zeros(Nfactors, numbofparts, top_trunc+size(rho, 3));
        rhotemp(:, :, 1:size(rho, 3)) = rho;
        rho = rhotemp;
        rho_idiotemp = zeros(p, numbofparts, top_trunc+size(rho_idio, 3));
        rho_idiotemp(:, :, 1:size(rho_idio, 3)) = rho_idio;
        rho_idio = rho_idiotemp;
        logprob_margtemp = zeros(trainT, top_trunc+size(logprob_marg, 2), numbofparts);
        logprob_margtemp(:, 1:size(logprob_marg, 2), :) = logprob_marg;
        logprob_marg = logprob_margtemp;
        logprob_jointtemp = zeros(trainT, top_trunc+size(logprob_joint, 2), numbofparts);
        logprob_jointtemp(:, 1:size(logprob_joint, 2), :) = logprob_joint;
        logprob_joint = logprob_jointtemp;
    end
    
    newloglike = zeros(1, numbofparts);
    mu(:, :, it) = ((ones(numbofparts, 1) * mu0)' + (sqrt(ones(numbofparts, 1) * Sigma0))' .* randn(p, numbofparts));
    for i = 1:p
        for part = 1:numbofparts
            S(i, part, it) = 1 / gamrnd(Salpha, 1 / ((Salpha - 1) * Smu(i, part)));
        end
    end
    
    for part = 1:numbofparts
        check2 = 0;
        while ( check2 == 0 )
            check2 = 1;
            
            delta(1, part, it) = gamrnd(a1, 1);
            tau(1, part, it) = delta(1, part, it);
            delta(2:end, part, it) = gamrnd(a2, 1, Nfactors - 1, 1);
            for i = 2:Nfactors
                tau(i, part, it) = tau(i - 1, part, it) * delta(i, part, it);
            end
            
            for k = 1:Nfactors
                for j = 1:p
                    phi(j, k, part, it) = gamrnd(nu / 2, 2 / nu);
                    lambda(j, k, part, it) = randn / sqrt(tau(k, part, it) * phi(j, k, part, it));
                end
                rho(k, part, it) = 0;
                while ( (rho(k, part, it) < rhoepsilon) || (rho(k, part, it) > 1 - rhoepsilon) )
                    rho(k, part, it) = betarnd(1, 1);
                end
            end
            for i = 1:p
                phi_idio(i, part, it) = gamrnd(nu / 2, 2 / nu);
                rho_idio(i, part, it) = 0;
                while ( (rho_idio(i, part, it) < rhoepsilon) || (rho_idio(i, part, it) > 1 - rhoepsilon) )
                    rho_idio(i, part, it) = betarnd(1, 1);
                end
            end
            
            for k = 1:Nfactors
                for i1 = 1:(lags + 1)
                    for j1 = 1:(lags + 1)
                        SuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p), part, it) = SuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p), part, it) + rho(k, part, it)^(abs(i1 - j1)) * (lambda(:, k, part, it) * lambda(:, k, part, it)');
                    end
                end
            end
            for k = 1:p
                for i1 = 1:(lags + 1)
                    for j1 = 1:(lags + 1)
                        SuperP((i1 - 1) * p + k, (j1 - 1) * p + k, part, it) = SuperP((i1 - 1) * p + k, (j1 - 1) * p + k, part, it) + rho_idio(k, part, it)^(abs(i1 - j1)) / phi_idio(k, part, it);
                    end
                end
            end
            P(:, :, part, it) = SuperP(1:(lags*p), 1:(lags*p), part, it);
            
            sumlogS = sum(log(S(:, part, it)));
            for j1 = 1:lags
                diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu(:, part, it)') ./ (ones(trainT - lags, 1) * S(:, part, it)');
            end
            diffstar2 = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu(:, part, it)') ./ (ones(trainT - lags, 1) * S(:, part, it)');
            [logprob_marg((lags+1):trainT, it, part), check] = logmvnpdf(diffstar1, zeros(1, lags * p), P(:, :, part, it));
            if ( check2 == 1 )
                check2 = check;
            end
            logprob_marg((lags+1):trainT, it, part) = logprob_marg((lags+1):trainT, it, part) - lags * sumlogS;
            [logprob_joint((lags+1):trainT, it, part), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), SuperP(:, :, part, it));
            if ( check2 == 1 )
                check2 = check;
            end
            logprob_joint((lags+1):trainT, it, part) = logprob_joint((lags+1):trainT, it, part) - (lags + 1) * sumlogS;
        end
    end
    V(:, it) = betarnd(1, M);
    for part = 1:numbofparts
        while ( (V(part, it) < Vepsilon) || (V(part, it) > 1 - Vepsilon) )
            V(part, it) = betarnd(1, M(part));
        end
    end
    
    for part = 1:numbofparts
        prodV = prod(1 - V(part, 1:(it-1)));
        temp1 = prodV * V(part, it);
        
        prob(part, 1:(it - 1)) = prob(part, 1:(it - 1)) * (1 - prodV) / (1 - prodV + temp1);
        prob(part, it) = temp1 / (1 - prodV + temp1);
        
        z1 = prob(part, 1:it) > 0;
        maxstar = max(permute(logprob_joint((lags+1):trainT, z1==1, part), [2 1 3]))';
        newloglike(part) = ones(1, trainT - lags) * (log(exp(logprob_joint((lags+1):trainT, z1==1, part) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
        maxstar = max(permute(logprob_marg((lags+1):trainT, z1==1, part), [2 1 3]))';
        newloglike(part) = newloglike(part) - ones(1, trainT - lags) * (log(exp(logprob_marg((lags+1):trainT, z1==1, part) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
        
        logweight(part) = newloglike(part) - loglike(part);
        
        if ( (isnan(logweight(part)) == 1) || (isinf(logweight(part)) == 1) )
            stop;
        end
        if ( (isnan(newloglike(part)) == 1) || (isinf(newloglike(part)) == 1) )
            stop;
        end
        if ( (isnan(loglike(part)) == 1) || (isinf(loglike(part)) == 1) )
            stop;
        end
        if ( (sum(isnan(prob(part, 1:it))) > 0) || (sum(isinf(prob(part, 1:it))) > 0) )
            stop;
        end
        
    end
    
    
    ESS = (sum(exp(logweight - max(logweight))))^2 / (sum(exp(2*(logweight - max(logweight)))));
%     
%     disp(num2str(it));
%     disp(['ESS = ' num2str(ESS)]);
%     disp(' ');
    
    rat(it - start_trunc + 1) = mean(exp(logweight));
    storeESS(it - start_trunc + 1) = ESS;
    
    if ( ESS < 0.7 * numbofparts )
        prob2 = exp(logweight - max(logweight));
        fprob = cumsum(prob2);
        fprob = fprob / fprob(end);
        
        u1 = rand / numbofparts;
        j = 1;
        count =  1;
        while ( j <= numbofparts )
            while ( (u1 < fprob(j)) && (j <= numbofparts) )
                partstar(count) = j;
                u1 = u1 + 1 / numbofparts;
                count = count + 1;
            end
            j = j + 1;
        end
        
        mu(:, :, 1:it) = mu(:, partstar, 1:it);
        S(:, :, 1:it) = S(:, partstar, 1:it);
        P(:, :, :, 1:it) = P(:, :, partstar, 1:it);
        SuperP(:, :, :, 1:it) = SuperP(:, :, partstar, 1:it);
        logprob_marg(:, 1:it, :) = logprob_marg(:, 1:it, partstar);
        logprob_joint(:, 1:it, :) = logprob_joint(:, 1:it, partstar);
        rho(:, :, 1:it) = rho(:, partstar, 1:it);
        rho_idio(:, :, 1:it) = rho_idio(:, partstar, 1:it);
        lambda(:, :, :, 1:it) = lambda(:, :, partstar, 1:it);
        phi(:, :, :, 1:it) = phi(:, :, partstar, 1:it);
        phi_idio(:, :, 1:it) = phi_idio(:, partstar, 1:it);
        tau(:, :, 1:it) = tau(:, partstar, 1:it);
        delta( :, :, 1:it) = delta(:, partstar, 1:it);
        V(:, 1:it) = V(partstar, 1:it);
        prob(:, 1:it) = prob(partstar, 1:it);
        loglike = newloglike(partstar);
        M = M(partstar);
        Smu = Smu(:, partstar);
        
        logweight = zeros(1, numbofparts);
        
        for it3 = 1:1
            
            for part = 1:numbofparts
                
                it2 = it2 + 1;
                
                % Update mu
                for comp = 1:it
                    for i = 1:p
                        newmu = mu(:, part, comp);
                        newmu(i) = newmu(i) + exp(logmusd(i)) * randn;
                        
                        newlogprob_marg = logprob_marg(:, 1:it, part);
                        newlogprob_joint = logprob_joint(:, 1:it, part);
                        
                        sumlogS = sum(log(S(:, part, comp)));
                        for j1 = 1:lags
                            diffstar1(:, ((j1-1)*p+1):(j1*p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * newmu') ./ (ones(trainT - lags, 1) * S(:, part, comp)');
                        end
                        diffstar2 = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * newmu') ./ (ones(trainT - lags, 1) * S(:, part, comp)');
                        
                        check2 = 1;
                        [newlogprob_marg((lags+1):trainT, comp, 1), check] = logmvnpdf(diffstar1, zeros(1, lags * p), P(:, :, part, comp));
                        if ( check2 == 1 )
                            check2 = check;
                        end
                        newlogprob_marg((lags+1):trainT, comp, 1) = newlogprob_marg((lags+1):trainT, comp, 1) - lags * sumlogS;
                        [newlogprob_joint((lags+1):trainT, comp, 1), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), SuperP(:, :, part, comp));
                        if ( check2 == 1 )
                            check2 = check;
                        end
                        newlogprob_joint((lags+1):trainT, comp, 1) = newlogprob_joint((lags+1):trainT, comp, 1) - (lags + 1) * sumlogS;
                        
                        if ( check2 == 1 )
                            z1 = prob(part, 1:it) > 0;
                            maxstar = max(newlogprob_joint((lags+1):trainT, z1==1, 1), [], 2);
                            newloglike = ones(1, trainT - lags) * (log(exp(newlogprob_joint((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                            maxstar = max(newlogprob_marg((lags+1):trainT, z1==1, 1), [], 2);
                            newloglike = newloglike - ones(1, trainT - lags) * (log(exp(newlogprob_marg((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                            
                            logaccept = newloglike - loglike(part);
                            logaccept = logaccept - 0.5 * (newmu(i) - mu0(i))^2 / Sigma0(i);
                            logaccept = logaccept + 0.5 * (mu(i, part, comp) - mu0(i))^2 / Sigma0(i);
                            
                            accept = 1;
                            if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                                accept = 0;
                            elseif ( logaccept < 0 )
                                accept = exp(logaccept);
                            end
                        else
                            accept = 0;
                        end
                        
                        muaccept(i) = muaccept(i) + accept;
                        mucount(i) = mucount(i) + 1;
                        
                        if ( rand < accept )
                            mu(i, part, comp) = newmu(i);
                            loglike(part) = newloglike;
                            logprob_marg(:, comp, part) = newlogprob_marg(:, comp, 1);
                            logprob_joint(:, comp, part) = newlogprob_joint(:, comp, 1);
                        end
                        
                        logmusd(i) = logmusd(i) + 1 / it2^0.55 * (accept - 0.3);
                    end
                end
                
                
                
                
                
                
                
                % Update lambda
                for comp = 1:it
                    for k = 1:Nfactors
                        
                        newlambda = lambda(:, :, part, comp);
                        
                        var1 = (lambdasum2 - lambdasum1 * lambdasum1' / lambdasum3) / (lambdasum3 - 1);
                        newlambda(:, k) = newlambda(:, k) + exp(loglambdasd2) * chol(var1)' * randn(p, 1);
                        
                        newSuperP = SuperP(:, :, part, comp);
                        for i1 = 1:(lags + 1)
                            for j1 = 1:(lags + 1)
                                newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) - rho(k, part, comp)^(abs(i1 - j1)) * (lambda(:, k, part, comp) * lambda(:, k, part, comp)');
                                newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) + rho(k, part, comp)^(abs(i1 - j1)) * (newlambda(:, k) * newlambda(:, k)');
                            end
                        end
                        
                        newP = newSuperP(1:(lags*p), 1:(lags*p));
                        
                        newlogprob_marg = logprob_marg(:, 1:it, part);
                        newlogprob_joint = logprob_joint(:, 1:it, part);
                        
                        sumlogS = sum(log(S(:, part, comp)));
                        for j1 = 1:lags
                            diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu(:, part, comp)') ./ (ones(trainT - lags, 1) * S(:, part, comp)');
                        end
                        diffstar2 = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu(:, part, comp)') ./ (ones(trainT - lags, 1) * S(:, part, comp)');
                        check2 = 1;
                        [newlogprob_marg((lags+1):trainT, comp, 1), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                        if ( check2 == 1 )
                            check2 = check;
                        end
                        newlogprob_marg((lags+1):trainT, comp, 1) = newlogprob_marg((lags+1):trainT, comp, 1) - lags * sumlogS;
                        [newlogprob_joint((lags+1):trainT, comp, 1), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                        if ( check2 == 1 )
                            check2 = check;
                        end
                        newlogprob_joint((lags+1):trainT, comp, 1) = newlogprob_joint((lags+1):trainT, comp, 1) - (lags + 1) * sumlogS;
                        
                        if ( check2 ==  1 )
                            z1 = prob(part, 1:it) > 0;
                            maxstar = max(newlogprob_joint((lags+1):trainT, z1==1, 1), [], 2);
                            newloglike = ones(1, trainT - lags) * (log(exp(newlogprob_joint((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                            maxstar = max(newlogprob_marg((lags+1):trainT, z1==1, 1), [], 2);
                            newloglike = newloglike - ones(1, trainT - lags) * (log(exp(newlogprob_marg((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                            
                            logaccept = newloglike - loglike(part);
                            logaccept = logaccept - 0.5 * sum(newlambda(:, k).^2 .* tau(k, part, comp) .* phi(:, k, part, comp));
                            logaccept = logaccept + 0.5 * sum(lambda(:, k, part, comp).^2 .* tau(k, part, comp) .* phi(:, k, part, comp));
                            
                            accept = 1;
                            if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                                accept = 0;
                            elseif ( logaccept < 0 )
                                accept = exp(logaccept);
                            end
                        else
                            accept = 0;
                        end
                        
                        lambdaaccept2 = lambdaaccept2 + accept;
                        lambdacount2 = lambdacount2 + 1;
                        
                        if ( rand < accept )
                            lambda(:, :, part, comp) = newlambda;
                            SuperP(:, :, part, comp) = newSuperP;
                            P(:, :, part, comp) = newP;
                            loglike(part) = newloglike;
                            logprob_marg(:, comp, part) = newlogprob_marg(:, comp, 1);
                            logprob_joint(:, comp, part) = newlogprob_joint(:, comp, 1);
                        end
                        
                        loglambdasd2 = loglambdasd2 + 1 / it2^0.55 * (accept - 0.3);
                        
                        lambdasum1 = lambdasum1 + lambda(:, k, part, comp);
                        lambdasum2 = lambdasum2 + lambda(:, k, part, comp) * lambda(:, k, part, comp)';
                        lambdasum3 = lambdasum3 + 1;
                        
                    end
                end
                
                
                
                
                
                
                
                % Update phi_idio
                for comp = 1:it
                    
                    newtrans = log(phi_idio(:, part, comp));
                    var1 = (phi_idiosum2 - phi_idiosum1 * phi_idiosum1' / phi_idiosum3) / (phi_idiosum3 - 1);
                    newtrans = newtrans + exp(logphi_idiosd2) * chol(var1)' * randn(p, 1);
                    newphi_idio = exp(newtrans);
                    
                    newSuperP = SuperP(:, :, part, comp);
                    for i1 = 1:(lags + 1)
                        for j1 = 1:(lags + 1)
                            for k = 1:p
                                newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) = newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) - rho_idio(k, part, comp)^(abs(i1 - j1)) / phi_idio(k, part, comp);
                                newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) = newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) + rho_idio(k, part, comp)^(abs(i1 - j1)) / newphi_idio(k);
                            end
                        end
                    end
                    
                    newP = newSuperP(1:(lags*p), 1:(lags*p));
                    
                    newlogprob_marg = logprob_marg(:, 1:it, part);
                    newlogprob_joint = logprob_joint(:, 1:it, part);
                    
                    sumlogS = sum(log(S(:, part, comp)));
                    for j1 = 1:lags
                        diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu(:, part, comp)') ./ (ones(trainT - lags, 1) * S(:, part, comp)');
                    end
                    diffstar2 = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu(:, part, comp)') ./ (ones(trainT - lags, 1) * S(:, part, comp)');
                    check2 = 1;
                    [newlogprob_marg((lags+1):trainT, comp, 1), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                    if ( check2 == 1 )
                        check2 = check;
                    end
                    newlogprob_marg((lags+1):trainT, comp, 1) = newlogprob_marg((lags+1):trainT, comp, 1) - lags * sumlogS;
                    [newlogprob_joint((lags+1):trainT, comp, 1), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                    if ( check2 == 1 )
                        check2 = check;
                    end
                    newlogprob_joint((lags+1):trainT, comp, 1) = newlogprob_joint((lags+1):trainT, comp, 1) - (lags + 1) * sumlogS;
                    
                    if ( check2 == 1 )
                        z1 = prob(part, 1:it) > 0;
                        maxstar = max(newlogprob_joint((lags+1):trainT, z1==1, 1), [], 2);
                        newloglike = ones(1, trainT - lags) * (log(exp(newlogprob_joint((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                        maxstar = max(newlogprob_marg((lags+1):trainT, z1==1, 1), [], 2);
                        newloglike = newloglike - ones(1, trainT - lags) * (log(exp(newlogprob_marg((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                        
                        logaccept = newloglike - loglike(part);
                        logaccept = logaccept + 0.5 * nu * (sum(log(newphi_idio)) - sum(log(phi_idio(:, part, comp))));
                        logaccept = logaccept - 0.5 * nu * (sum(newphi_idio) - sum(phi_idio(:, part, comp)));
                        
                        accept = 1;
                        if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                            accept = 0;
                        elseif ( logaccept < 0 )
                            accept = exp(logaccept);
                        end
                    else
                        accept = 0;
                    end
                    
                    phi_idioaccept2 = phi_idioaccept2 + accept;
                    phi_idiocount2 = phi_idiocount2 + 1;
                    
                    if ( rand < accept )
                        phi_idio(:, part, comp) = newphi_idio;
                        SuperP(:, :, part, comp) = newSuperP;
                        P(:, :, part, comp) = newP;
                        loglike(part) = newloglike;
                        logprob_marg(:, comp, part) = newlogprob_marg(:, comp, 1);
                        logprob_joint(:, comp, part) = newlogprob_joint(:, comp, 1);
                    end
                    
                    logphi_idiosd2 = logphi_idiosd2 + 1 / it2^0.55 * (accept - 0.3);
                    
                    xtemp = log(phi_idio(:, part, comp));
                    phi_idiosum1 = phi_idiosum1 + xtemp;
                    phi_idiosum2 = phi_idiosum2 + xtemp * xtemp';
                    phi_idiosum3 = phi_idiosum3 + 1;
                end
                
                
                
                
                % Update rho
                for comp = 1:it
                    
                    newtrans = log(rho(:, part, comp) - rhoepsilon) - log(1 - rhoepsilon - rho(:, part, comp));
                    var1 = (rhosum2 - rhosum1 * rhosum1' / rhosum3) / (rhosum3 - 1);
                    
                    newtrans = newtrans + exp(logrhosd2) * chol(var1)' * randn(p, 1);
                    newrho = rhoepsilon + (1 - 2 * rhoepsilon) * exp(newtrans) ./ (exp(newtrans) + 1);
                    
                    newSuperP = SuperP(:, :, part, comp);
                    for i1 = 1:(lags + 1)
                        for j1 = 1:(lags + 1)
                            for k = 1:p
                                newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) - rho(k, part, comp)^(abs(i1 - j1)) * (lambda(:, k, part, comp) * lambda(:, k, part, comp)');
                                newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) = newSuperP((i1 - 1) * p + (1:p), (j1 - 1) * p + (1:p)) + newrho(k)^(abs(i1 - j1)) * (lambda(:, k, part, comp) * lambda(:, k, part, comp)');
                            end
                        end
                    end
                    newP = newSuperP(1:(lags*p), 1:(lags*p));
                    
                    newlogprob_marg = logprob_marg(:, 1:it, part);
                    newlogprob_joint = logprob_joint(:, 1:it, part);
                    
                    sumlogS = sum(log(S(:, part, comp)));
                    for j1 = 1:lags
                        diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu(:, part, comp)') ./ (ones(trainT - lags, 1) * S(:, part, comp)');
                    end
                    diffstar2 = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu(:, part, comp)') ./ (ones(trainT - lags, 1) * S(:, part, comp)');
                    check2 = 1;
                    [newlogprob_marg((lags+1):trainT, comp, 1), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                    if ( check2 == 1 )
                        check2 = check;
                    end
                    newlogprob_marg((lags+1):trainT, comp, 1) = newlogprob_marg((lags+1):trainT, comp, 1) - lags * sumlogS;
                    [newlogprob_joint((lags+1):trainT, comp, 1), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                    if ( check2 == 1 )
                        check2 = check;
                    end
                    newlogprob_joint((lags+1):trainT, comp, 1) = newlogprob_joint((lags+1):trainT, comp, 1) - (lags + 1) * sumlogS;
                    
                    if ( check2 == 1 )
                        z1 = prob(part, 1:it) > 0;
                        maxstar = max(newlogprob_joint((lags+1):trainT, z1==1, 1), [], 2);
                        newloglike = ones(1, trainT - lags) * (log(exp(newlogprob_joint((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                        maxstar = max(newlogprob_marg((lags+1):trainT, z1==1, 1), [], 2);
                        newloglike = newloglike - ones(1, trainT - lags) * (log(exp(newlogprob_marg((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                        
                        logaccept = newloglike - loglike(part);
                        logaccept = logaccept - sum(log(1 ./ (newrho - rhoepsilon) + 1 ./ (1 - newrho - rhoepsilon)));
                        logaccept = logaccept + sum(log(1 ./ (rho(:, part, comp) - rhoepsilon) + 1 ./ (1 - rhoepsilon - rho(:, part, comp))));
                        
                        accept = 1;
                        if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                            accept = 0;
                        elseif ( logaccept < 0 )
                            accept = exp(logaccept);
                        end
                    else
                        accept = 0;
                    end
                    
                    rhoaccept2 = rhoaccept2 + accept;
                    rhocount2 = rhocount2 + 1;
                    
                    if ( rand < accept )
                        rho(:, part, comp) = newrho;
                        SuperP(:, :, part, comp) = newSuperP;
                        P(:, :, part, comp) = newP;
                        loglike(part) = newloglike;
                        logprob_marg(:, comp, part) = newlogprob_marg(:, comp, 1);
                        logprob_joint(:, comp, part) = newlogprob_joint(:, comp, 1);
                    end
                    
                    xtemp = log(rho(:, part, comp) - rhoepsilon) - log(1 - rhoepsilon - rho(:, part, comp));
                    rhosum1 = rhosum1 + xtemp;
                    rhosum2 = rhosum2 + xtemp * xtemp';
                    rhosum3 = rhosum3 + 1;
                    
                    logrhosd2 = logrhosd2 + 1 / it2^0.55 * (accept - 0.3);
                end
                
                
                
                
                
                
                
                
                % Update rho_idio
                for comp = 1:it
                    
                    newtrans = log(rho_idio(:, part, comp) - rhoepsilon) - log(1 - rhoepsilon - rho_idio(:, part, comp));
                    var1 = (rho_idiosum2 - rho_idiosum1 * rho_idiosum1' / rho_idiosum3) / (rho_idiosum3 - 1);
                    
                    newtrans = newtrans + exp(logrho_idiosd2) * chol(var1)' * randn(p, 1);
                    newrho_idio = rhoepsilon + (1 - 2 * rhoepsilon) * exp(newtrans) ./ (exp(newtrans) + 1);
                    
                    newSuperP = SuperP(:, :, part, comp);
                    for i1 = 1:(lags + 1)
                        for j1 = 1:(lags + 1)
                            for k = 1:p
                                newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) = newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) - rho_idio(k, part, comp)^(abs(i1 - j1)) / phi_idio(k, part, comp);
                                newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) = newSuperP((i1 - 1) * p + k, (j1 - 1) * p + k) + newrho_idio(k)^(abs(i1 - j1)) / phi_idio(k, part, comp);
                            end
                        end
                    end
                    newP = newSuperP(1:(lags*p), 1:(lags*p));
                    
                    newlogprob_marg = logprob_marg(:, 1:it, part);
                    newlogprob_joint = logprob_joint(:, 1:it, part);
                    
                    sumlogS = sum(log(S(:, part, comp)));
                    for j1 = 1:lags
                        diffstar1(:, ((j1 - 1) * p + 1):(j1 * p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu(:, part, comp)') ./ (ones(trainT - lags, 1) * S(:, part, comp)');
                    end
                    diffstar2 = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu(:, part, comp)') ./ (ones(trainT - lags, 1) * S(:, part, comp)');
                    check2 = 1;
                    [newlogprob_marg((lags+1):trainT, comp, 1), check] = logmvnpdf(diffstar1, zeros(1, lags * p), newP);
                    if ( check2 == 1 )
                        check2 = check;
                    end
                    newlogprob_marg((lags+1):trainT, comp, 1) = newlogprob_marg((lags+1):trainT, comp, 1) - lags * sumlogS;
                    [newlogprob_joint((lags+1):trainT, comp, 1), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), newSuperP);
                    if ( check2 == 1 )
                        check2 = check;
                    end
                    newlogprob_joint((lags+1):trainT, comp, 1) = newlogprob_joint((lags+1):trainT, comp, 1) - (lags + 1) * sumlogS;
                    
                    if ( check2 == 1 )
                        z1 = prob(part, 1:it) > 0;
                        maxstar = max(newlogprob_joint((lags+1):trainT, z1==1, 1), [], 2);
                        newloglike = ones(1, trainT - lags) * (log(exp(newlogprob_joint((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                        maxstar = max(newlogprob_marg((lags+1):trainT, z1==1, 1), [], 2);
                        newloglike = newloglike - ones(1, trainT - lags) * (log(exp(newlogprob_marg((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                        
                        logaccept = newloglike - loglike(part);
                        logaccept = logaccept - sum(log(1 ./ (newrho_idio - rhoepsilon) + 1 ./ (1 - rhoepsilon - newrho_idio)));
                        logaccept = logaccept + sum(log(1 ./ (rho_idio(:, part, comp) - rhoepsilon) + 1 ./ (1 - rhoepsilon - rho_idio(:, part, comp))));
                        
                        accept = 1;
                        if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                            accept = 0;
                        elseif ( logaccept < 0 )
                            accept = exp(logaccept);
                        end
                    else
                        accept = 0;
                    end
                    
                    rho_idioaccept2 = rho_idioaccept2 + accept;
                    rho_idiocount2 = rho_idiocount2 + 1;
                    
                    if ( rand < accept )
                        rho_idio(:, part, comp) = newrho_idio;
                        SuperP(:, :, part, comp) = newSuperP;
                        P(:, :, part, comp) = newP;
                        loglike(part) = newloglike;
                        logprob_marg(:, comp, part) = newlogprob_marg(:, comp, 1);
                        logprob_joint(:, comp, part) = newlogprob_joint(:, comp, 1);
                    end
                    
                    logrho_idiosd2 = logrho_idiosd2 + 1 / it2^0.55 * (accept - 0.3);
                    
                    xtemp = log(rho_idio(:, part, comp) - rhoepsilon) - log(1 - rhoepsilon - rho_idio(:, part, comp));
                    rho_idiosum1 = rho_idiosum1 + xtemp;
                    rho_idiosum2 = rho_idiosum2 + xtemp * xtemp';
                    rho_idiosum3 = rho_idiosum3 + 1;
                end
                
                
                
                
                % Update phi
                for comp = 1:it
                    for k = 1:Nfactors
                        phi(:, k, part, comp) = gamrnd((nu + 1) / 2, 2 ./ (nu + tau(k, part, comp) * lambda(:, k, part, comp).^2));
                    end
                    
                    % Update delta
                    sum1 = zeros(Nfactors, 1);
                    for k = 1:Nfactors
                        sum1(k) = sum(phi(:, k, part, comp) .* lambda(:, k, part, comp).^2);
                    end
                    
                    olddelta = delta(1, part, comp);
                    delta(1, part, comp) = gamrnd(a1 + p * Nfactors / 2, 1 / (1 + 0.5 * sum(tau(1:Nfactors, part, comp) / delta(1, part, comp) .* sum1)));
                    tau(1, part, comp) = delta(1, part, comp);
                    tau(2:Nfactors, part, comp) = tau(2:Nfactors, part, comp) * delta(1, part, comp) / olddelta;
                    for k = 2:Nfactors
                        olddelta = delta(k, part, comp);
                        delta(k, part, comp) = gamrnd(a1 + p * (Nfactors - k + 1) / 2, 1 / (1 + 0.5 * sum(tau(k:Nfactors, part, comp) / delta(k, part, comp) .* sum1(k:Nfactors))));
                        tau(k, part, comp) = tau(k - 1, part, comp) * delta(k, part, comp);
                        tau((k+1):Nfactors, part, comp) = tau((k+1):Nfactors, part, comp) * delta(k, part, comp) / olddelta;
                    end
                end
                                
                % Update S
                for comp = 1:it
                    for i = 1:p
                        newS = S(:, part, comp);
                        newS(i) = newS(i) * exp(exp(logSsd(i)) * randn);
                        
                        newlogprob_marg = logprob_marg(:, 1:it, part);
                        newlogprob_joint = logprob_joint(:, 1:it, part);
                        
                        sumlogS = sum(log(newS));
                        for j1 = 1:lags
                            diffstar1(:, ((j1-1)*p+1):(j1*p)) = (y_lags(1:(trainT-lags), ((j1-1)*p+1):(j1*p)) - ones(trainT - lags, 1) * mu(:, part, comp)') ./ (ones(trainT - lags, 1) * newS');
                        end
                        diffstar2 = (y_cur(1:(trainT-lags), :) - ones(trainT - lags, 1) * mu(:, part, comp)') ./ (ones(trainT - lags, 1) * newS');
                        
                        check2 = 1;
                        [newlogprob_marg((lags+1):trainT, comp, 1), check] = logmvnpdf(diffstar1, zeros(1, lags * p), P(:, :, part, comp));
                        if ( check2 == 1 )
                            check2 = check;
                        end
                        newlogprob_marg((lags+1):trainT, comp, 1) = newlogprob_marg((lags+1):trainT, comp, 1) - lags * sumlogS;
                        [newlogprob_joint((lags+1):trainT, comp, 1), check] = logmvnpdf([diffstar2 diffstar1], zeros(1, (lags + 1) * p), SuperP(:, :, part, comp));
                        if ( check2 == 1 )
                            check2 = check;
                        end
                        newlogprob_joint((lags+1):trainT, comp, 1) = newlogprob_joint((lags+1):trainT, comp, 1) - (lags + 1) * sumlogS;
                        
                        if ( check2 == 1 )
                            z1 = prob(part, 1:it) > 0;
                            maxstar = max(newlogprob_joint((lags+1):trainT, z1==1, 1), [], 2);
                            newloglike = ones(1, trainT - lags) * (log(exp(newlogprob_joint((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                            maxstar = max(newlogprob_marg((lags+1):trainT, z1==1, 1), [], 2);
                            newloglike = newloglike - ones(1, trainT - lags) * (log(exp(newlogprob_marg((lags+1):trainT, z1==1, 1) - maxstar * ones(1, sum(z1))) * prob(part, z1==1)') + maxstar);
                            
                            logaccept = newloglike - loglike(part);
                            logaccept = logaccept - (Salpha + 1) * log(newS(i)) - (Salpha - 1) * Smu(i, part) / newS(i);
                            logaccept = logaccept + (Salpha + 1) * log(S(i, part, comp)) + (Salpha - 1) * Smu(i, part) / S(i, part, comp);
                            logaccept = logaccept + log(newS(i)) - log(S(i, part, comp));
                            
                            accept = 1;
                            if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                                accept = 0;
                            elseif ( logaccept < 0 )
                                accept = exp(logaccept);
                            end
                        else
                            accept = 0;
                        end
                        
                        Saccept(i) = Saccept(i) + accept;
                        Scount(i) = Scount(i) + 1;
                        
                        if ( rand < accept )
                            S(i, part, comp) = newS(i);
                            loglike(part) = newloglike;
                            logprob_marg(:, comp, part) = newlogprob_marg(:, comp, 1);
                            logprob_joint(:, comp, part) = newlogprob_joint(:, comp, 1);
                        end
                        
                        logSsd(i) = logSsd(i) + 1 / it2^0.55 * (accept - 0.3);
                    end
                end
                
                % Update Smu
                for i = 1:p
                    Smu(i, part) = gamrnd(it * Salpha + Smu_alpha, 1 / ((Salpha - 1) * sum(1./S(i, part, 1:it)) + Smu_beta));
                end
                
                % Update V
                for comp = 1:it
                    
                    newV = V(part, :);
                    trans = log(V(part, comp) - Vepsilon) - log(1 - Vepsilon - V(part, comp));
                    newtrans = trans + exp(logVsd) * randn;
                    newV(comp) = Vepsilon + (1 - 2 * Vepsilon) * exp(newtrans) / (1 + exp(newtrans));
                    
                    newprob = prob(part, 1:it);
                    prodV = 1;
                    for i = 1:it
                        newprob(i) = newV(i) * prodV;
                        prodV = prodV * (1 - newV(i));
                    end
                    newprob = newprob / sum(newprob);
                    
                    z1 = newprob > 0;
                    maxstar = max(permute(logprob_joint((lags+1):trainT, z1==1, part), [2 1 3]))';
                    newloglike = ones(1, trainT - lags) * (log(exp(logprob_joint((lags+1):trainT, z1==1, part) - maxstar * ones(1, sum(z1))) * newprob(z1==1)') + maxstar);
                    maxstar = max(permute(logprob_marg((lags+1):trainT, z1==1, part), [2 1 3]))';
                    newloglike = newloglike - ones(1, trainT - lags) * (log(exp(logprob_marg((lags+1):trainT, z1==1, part) - maxstar * ones(1, sum(z1))) * newprob(z1==1)') + maxstar);
                    
                    logaccept = newloglike - loglike(part);
                    logaccept = logaccept + (M(part) - 1) * (log(1 - newV(comp)) - log(1 - V(part, comp)));
                    logaccept = logaccept - log(1 / (newV(comp) - Vepsilon) + 1 / (1 - Vepsilon - newV(comp))) + log(1 / (V(part, comp) - Vepsilon) + 1 / (1 - Vepsilon - V(part, comp)));
                    
                    accept = 1;
                    if ( (isnan(logaccept) == 1) || (isinf(logaccept) == 1) || (isreal(logaccept) == 0) )
                        accept = 0;
                    elseif ( logaccept < 0 )
                        accept = exp(logaccept);
                    end
                    
                    Vaccept = Vaccept + accept;
                    Vcount = Vcount + 1;
                    
                    if ( rand < accept )
                        V(part, comp) = newV(comp);
                        prob(part, 1:it) = newprob;
                        loglike(part) = newloglike;
                    end
                    
                    logVsd = logVsd + 1 / it2^0.55 * (accept - 0.3);
                end
                
                betastar_M = (1 - sum(log(1 - V(part, 1:it))));
                M(part) = gamrnd(1 + it, 1 / betastar_M);
                
            end
        end
    end
    
%     disp(['mu accept = (' num2str(min(muaccept./mucount)) ', ' num2str(max(muaccept./mucount)) ')']);
%     minstar = min(rhoaccept2./rhocount2);
%     maxstar = max(rhoaccept2./rhocount2);
%     disp('rho2 accept = ');
%     disp(num2str(minstar));
%     disp(num2str(maxstar));
%     minstar = min(rho_idioaccept2/rho_idiocount2);
%     maxstar = max(rho_idioaccept2/rho_idiocount2);
%     disp('rho_idio accept2 = ');
%     disp(num2str(minstar));
%     disp(num2str(maxstar));
%     minstar = min(phi_idioaccept2/phi_idiocount2);
%     maxstar = max(phi_idioaccept2/phi_idiocount2);
%     disp('phi_idio accept2 = ');
%     disp(num2str(minstar));
%     disp(num2str(maxstar));
%     minstar = min(min(lambdaaccept2./lambdacount2));
%     maxstar = max(max(lambdaaccept2./lambdacount2));
%     disp('lambda2 accept = ');
%     disp(num2str(minstar));
%     disp(num2str(maxstar));
%     minstar = min(Saccept./Scount);
%     maxstar = max(Saccept./Scount);
%     disp('S accept = ')
%     disp(num2str(minstar));
%     disp(num2str(maxstar));
%     disp(['V accept = ' num2str(Vaccept/Vcount)]);
%     disp(' ');
    
    if ( it - start_trunc >= numb_trunc )
        x1 = (it - numb_trunc + 1 - start_trunc + 1):(it - start_trunc + 1);
        x2 = (it - numb_trunc - start_trunc + 1):(it - 1 - start_trunc + 1);
%         disp(storeESS(x1))
%         disp(storeESS(x2))
%         disp(storeESS(x1) - storeESS(x2))
        check_trunc = sum(abs(storeESS(x1) - storeESS(x2)) < epsilon_trunc * numbofparts) == numb_trunc;
%         disp(epsilon_trunc)
%         disp(numbofparts)
%         disp(numb_trunc)
    end
end


weight = exp(logweight - max(logweight));
weight = weight / sum(weight);

for part = 1:numbofparts
    logpost(part) = loglike(part);
    for  comp = 1:it
        logpost(part) = logpost(part) - 0.5 * sum(((mu(:, part, comp) - mu0').^2) ./ Sigma0');
        for k = 1:Nfactors
            logpost(part) = logpost(part) + sum(lognormpdf(lambda(:, k, part, comp), 0, 1 ./ sqrt(tau(k, part, comp) .* phi(:, k, part, comp))));
            logpost(part) = logpost(part) + sum(loggampdf(phi(:, k, part, comp), nu / 2, 2 / nu));
        end
        logpost(part) = logpost(part) + loggampdf(delta(1, part, comp), a1, 1);
        logpost(part) = logpost(part) + sum(loggampdf(delta(2:end, part, comp), a2, 1));
        logpost(part) = logpost(part) + sum(loggampdf(phi_idio(:, part, comp), nu / 2, 2 / nu));
        logpost(part) = logpost(part) + sum(loggampdf(1./S(:, part, comp), Salpha, 1./((Salpha-1) * Smu(:, part)))) - 2 * sum(log(S(:, part, comp)));
        logpost(part) = logpost(part) + logbetapdf(V(part, comp), 1, M(part));
    end
    logpost(part) = logpost(part) + sum(loggampdf(1./Smu(:, part), Smu_alpha, 1/Smu_beta) - 2 * log(Smu(:, part)));
    logpost(part) = logpost(part) - M(part);
end

extralogprob_joint_ind = zeros(numbofparts, it, length(hsteps), p, T - trainT);
extralogprob_pred_ind = zeros(numbofparts, it, length(hsteps), p, T - trainT);
extralogprob_marg = zeros(numbofparts, it, length(hsteps), T - trainT);
extralogprob_joint = zeros(numbofparts, it, length(hsteps), T - trainT);


part1 = zeros(p, p * lags, numbofparts, it);
var1 = zeros(p, p, numbofparts, it);
for part = 1:numbofparts
    for comp = 1:it
        part1(:, :, part, comp) = SuperP(1:p, (p+1):((lags+1)*p), part, comp) / (SuperP((p+1):((lags+1)*p), (p+1):((lags+1)*p), part, comp));
        var1(:, :, part, comp) = SuperP(1:p, 1:p, part, comp) - part1(:, :, part, comp) * SuperP((p+1):((lags+1)*p), 1:p, part, comp);
        var1(:, :, part, comp) = 0.5 * var1(:, :, part, comp) + 0.5 * var1(:, :, part, comp)';
    end
end

extra_mean1 = zeros(p, it, numbofparts, length(hsteps), T - trainT);
extra_marg = zeros(it, numbofparts, length(hsteps), T - trainT);

check1 = ones(numbofparts, length(hsteps));

for part = 1:numbofparts
    for step1 = 1:length(hsteps)
        
        curdata2 = zeros(T - trainT - hsteps(step1) + 1, lags * p);
        curdata3 = zeros(T - trainT - hsteps(step1) + 1, lags * p);
        curdata4 = zeros(T - trainT - hsteps(step1) + 1, lags * p);
        for i1 = (trainT+1):(T+1-hsteps(step1))
            curdata2(i1 - trainT, :) = y_lags(i1 - lags, :);
            curdata3(i1 - trainT, :) = y_lags(i1 - lags, :);
            curdata4(i1 - trainT, :) = y_lags(i1 - lags, :);
        end
        
        diff1 = zeros(T - trainT - hsteps(step1) + 1, lags * p);
        diff2 = zeros(T - trainT - hsteps(step1) + 1, lags * p);
        diff3 = zeros(T - trainT - hsteps(step1) + 1, lags * p);
        store_marg2 = zeros(T - trainT - hsteps(step1) + 1, it);
        store_marg3 = zeros(T - trainT - hsteps(step1) + 1, it);
        store_marg4 = zeros(T - trainT - hsteps(step1) + 1, it);
        eps1 = zeros(p, T - trainT - hsteps(step1) + 1, 1);
        eps2 = zeros(p, T - trainT - hsteps(step1) + 1, 1);
        eps3 = zeros(p, T - trainT - hsteps(step1) + 1, 1);
        
        for k1 = 1:(hsteps(step1) - 1)
            alloc1 = ones(T - trainT - hsteps(step1) + 1, 1);
            alloc2 = ones(T - trainT - hsteps(step1) + 1, 1);
            alloc3 = ones(T - trainT - hsteps(step1) + 1, 1);
            
            for comp = 1:it
                sumlogS = sum(log(S(:, part, comp)));
                for j1 = 1:lags
                    diff1(:, ((j1-1)*p+1):(j1*p)) = (curdata2(:, ((j1-1)*p+1):(j1*p)) - ones(T-trainT-hsteps(step1)+1, 1)*mu(:, part, comp)') ./ (ones(T-trainT-hsteps(step1)+1, 1) * S(:, part, comp)');
                    diff2(:, ((j1-1)*p+1):(j1*p)) = (curdata3(:, ((j1-1)*p+1):(j1*p)) - ones(T-trainT-hsteps(step1)+1, 1)*mu(:, part, comp)') ./ (ones(T-trainT-hsteps(step1)+1, 1) * S(:, part, comp)');
                    diff3(:, ((j1-1)*p+1):(j1*p)) = (curdata4(:, ((j1-1)*p+1):(j1*p)) - ones(T-trainT-hsteps(step1)+1, 1)*mu(:, part, comp)') ./ (ones(T-trainT-hsteps(step1)+1, 1) * S(:, part, comp)');
                end
                store_marg2(:, comp) = logmvnpdf(diff1, zeros(1, lags * p), P(:, :, part, comp)) - lags * sumlogS;
                store_marg3(:, comp) = logmvnpdf(diff2, zeros(1, lags * p), P(:, :, part, comp)) - lags * sumlogS;
                store_marg4(:, comp) = logmvnpdf(diff3, zeros(1, lags * p), P(:, :, part, comp)) - lags * sumlogS;
            end
            
            z1 = prob(part, 1:it) > 0;
            maxstar = max(store_marg2(:, z1==1), [], 2);
            probstar = zeros(T - trainT - hsteps(step1) + 1, it);
            probstar(:, z1==1) = exp(store_marg2(:, z1==1) - maxstar*ones(1, sum(z1))) .* (ones(T-trainT-hsteps(step1)+1, 1)*prob(part, z1==1));
            fprob = cumsum(probstar, 2);
            fprob = fprob ./ (fprob(:, end) * ones(1, it));
            
            for i1 = (trainT+1):(T+1-hsteps(step1))
                u = rand;
                while ( fprob(i1 - trainT, alloc1(i1 - trainT)) < u )
                    alloc1(i1 - trainT) = alloc1(i1 - trainT) + 1;
                end
            end
            
            maxstar = max(store_marg3(:, z1==1), [], 2);
            probstar = zeros(T - trainT - hsteps(step1) + 1, it);
            probstar(:, z1==1) = exp(store_marg3(:, z1==1) - maxstar*ones(1, sum(z1))) .* (ones(T-trainT-hsteps(step1)+1, 1)*prob(part, z1==1));
            fprob = cumsum(probstar, 2);
            fprob = fprob ./ (fprob(:, end) * ones(1, it));
            
            for i1 = (trainT+1):(T+1-hsteps(step1))
                u = rand;
                while ( fprob(i1 - trainT, alloc2(i1 - trainT)) < u )
                    alloc2(i1 - trainT) = alloc2(i1 - trainT) + 1;
                end
            end
            
            maxstar = max(store_marg4(:, z1==1), [], 2);
            probstar = zeros(T - trainT - hsteps(step1) + 1, it);
            probstar(:, z1==1) = exp(store_marg4(:, z1==1) - maxstar*ones(1, sum(z1))) .* (ones(T-trainT-hsteps(step1)+1, 1)*prob(part, z1==1));
            fprob = cumsum(probstar, 2);
            fprob = fprob ./ (fprob(:, end) * ones(1, it));
            
            for i1 = (trainT+1):(T+1-hsteps(step1))
                u = rand;
                while ( fprob(i1 - trainT, alloc3(i1 - trainT)) < u )
                    alloc3(i1 - trainT) = alloc3(i1 - trainT) + 1;
                end
            end
            
            for j1 = 1:lags
                diff1(:, ((j1 - 1) * p + 1):(j1 * p)) = (curdata2(:, ((j1-1)*p+1):(j1*p)) - permute(mu(:, part, alloc1), [3 1 2])) ./ permute(S(:, part, alloc1), [3 1 2]);
                diff2(:, ((j1 - 1) * p + 1):(j1 * p)) = (curdata3(:, ((j1-1)*p+1):(j1*p)) - permute(mu(:, part, alloc2), [3 1 2])) ./ permute(S(:, part, alloc2), [3 1 2]);
                diff3(:, ((j1 - 1) * p + 1):(j1 * p)) = (curdata4(:, ((j1-1)*p+1):(j1*p)) - permute(mu(:, part, alloc3), [3 1 2])) ./ permute(S(:, part, alloc3), [3 1 2]);
            end
            
            for i1 = (trainT+1):(T+1-hsteps(step1))
                mean1 = part1(:, :, part, alloc1(i1 - trainT)) * diff1(i1 - trainT, :)';
                [eps1(:, i1 - trainT), err] = mvnrnd_local(mean1, var1(:, :, part, alloc1(i1 - trainT)));
                if (err == 1)
                    stop;
                end
            end
            curdata2(:, (p+1):end) = curdata2(:, 1:(end-p));
            curdata2(:, 1:p) = (permute(mu(:, part, alloc1), [1 3 2]) + permute(S(:, part, alloc1), [1 3 2]) .* eps1)';
            
            for i1 = (trainT+1):(T+1-hsteps(step1))
                mean2 = part1(:, :, part, alloc2(i1 - trainT)) * diff2(i1 - trainT, :)';
                [eps2(:, i1 - trainT), err] = mvnrnd_local(mean2, var1(:, :, part, alloc2(i1 - trainT)));
                if ( err == 1)
                    stop;
                end
            end
            curdata3(:, (p+1):end) = curdata3(:, 1:(end-p));
            curdata3(:, 1:p) = (permute(mu(:, part, alloc2), [1 3 2]) + permute(S(:, part, alloc2), [1 3 2]) .* eps2)';
            
            for i1 = (trainT+1):(T+1-hsteps(step1))
                mean3 = part1(:, :, part, alloc3(i1 - trainT)) * diff3(i1 - trainT, :)';
                [eps3(:, i1 - trainT), err] = mvnrnd_local(mean3, var1(:, :, part, alloc2(i1 - trainT)));
                if ( err == 1)
                    stop;
                end
            end
            curdata4(:, (p+1):end) = curdata4(:, 1:(end-p));
            curdata4(:, 1:p) = (permute(mu(:, part, alloc3), [1 3 2]) + permute(S(:, part, alloc3), [1 3 2]) .* eps3)';
        end
        
        
        
        for comp = 1:it
            sumlogS = sum(log(S(:, part, comp)));
            
            for j1 = 1:lags
                diff1(:, ((j1 - 1) * p + 1):(j1 * p)) = (curdata2(:, ((j1-1)*p+1):(j1*p)) - ones(T-trainT-hsteps(step1)+1, 1)*mu(:, part, comp)') ./ (ones(T-trainT-hsteps(step1)+1, 1) * S(:, part, comp)');
            end
            diff2 = (data((trainT + hsteps(step1)):T, :) - ones(T-trainT-hsteps(step1)+1, 1)*mu(:, part, comp)') ./ (ones(T-trainT-hsteps(step1)+1, 1) * S(:, part, comp)');
            
            extra_mean1(:, comp, part, step1, 1:(T-trainT-hsteps(step1)+1)) = mu(:, part, comp) * ones(1, T - trainT - hsteps(step1) + 1) + diag(S(:, part, comp)) * part1(:, :, part, comp) * diff1';
            extra_marg(comp, part, step1, 1:(T-trainT-hsteps(step1)+1)) = logmvnpdf(diff1, zeros(1, lags * p), P(:, :, part, comp)) - lags * sumlogS;
            extralogprob_marg(part, comp, step1, 1:(T-trainT-hsteps(step1)+1)) = logmvnpdf(diff1, zeros(1, lags * p), P(:, :, part, comp)) - lags * sumlogS;
            extralogprob_joint(part, comp, step1, 1:(T-trainT-hsteps(step1)+1)) = logmvnpdf([diff2 diff1], zeros(1, (lags + 1) * p), SuperP(:, :, part, comp)) - (lags + 1) * sumlogS;
            
            mean1 = part1(:, :, part, comp) * diff1';
            for j = 1:p
                x1 = [j (p+1):((lags+1)*p)];
                extralogprob_joint_ind(part, comp, step1, j, 1:(T-trainT-hsteps(step1)+1)) = logmvnpdf([diff2(:, j) diff1], zeros(1, 1 + lags * p), SuperP(x1, x1, part, comp)) - lags * sumlogS - log(S(j, part, comp));
                extralogprob_pred_ind(part, comp, step1, j, 1:(T-trainT-hsteps(step1)+1)) = logmvnpdf(diff2(:, j), mean1(j, :)', var1(j, j, part, comp)) - log(S(j, part, comp));
                
                if ( sum(isinf(extralogprob_pred_ind(part, comp, step1, j, 1:(T-trainT-hsteps(step1)+1)))) > 0 )
                    stop;
                end
                
            end
        end
    end
end


logpred = zeros(length(hsteps), numbofparts);
logpred_t = zeros(length(hsteps), T, numbofparts);
logpred_ind = zeros(length(hsteps), numbofparts, p);
logpred_ind_t = zeros(p, length(hsteps), T, numbofparts);

for step1 = 1:length(hsteps)
    for t = 1:(T + 1 - hsteps(step1) - trainT)
        temp1 = zeros(1, numbofparts);
        temp2 = zeros(p, numbofparts);
        
        for part = 1:numbofparts
            if ( check1(part, step1) == 1 )
                z1 = prob(part, 1:it) > 0;
                maxstar = max(extralogprob_joint(part, z1==1, step1, t), [], 2);
                temp1(part) = (log(sum(prob(part, z1==1) .* exp(extralogprob_joint(part, z1==1, step1, t) - maxstar*ones(1, sum(z1))))) + maxstar)';
            end
        end
        
        
        for part = 1:numbofparts
            if ( check1(part, step1) == 1 )
                z1 = prob(part, 1:it) > 0;
                for j = 1:p
                    maxstar2 = max(extralogprob_marg(part, z1==1, step1, t) + extralogprob_pred_ind(part, z1==1, step1, j, t), [], 2);
                    temp2(j, part) = (log(sum(prob(part, z1==1) .* exp(extralogprob_marg(part, z1==1, step1, t) + extralogprob_pred_ind(part, z1==1, step1, j, t) - maxstar2*ones(1, sum(z1))), 2)) + maxstar2)';
                    
                    if ( isinf(temp2(j, part)) == 1 )
                        stop;
                    end
                end
            end
        end
        
        for part = 1:numbofparts
            z1 = prob(part, 1:it) > 0;
            
            maxstar = max(extralogprob_marg(part, z1==1, step1, t), [], 2);
            temp1(part) = temp1(part) - (log(sum(prob(part, z1==1) .* exp(extralogprob_marg(part, z1==1, step1, t) - maxstar*ones(1, sum(z1))), 2)) + maxstar)';
            for j = 1:p
                temp2(j, part) = temp2(j, part) - (log(sum(prob(part, z1==1) .* exp(extralogprob_marg(part, z1==1, step1, t) - maxstar*ones(1, sum(z1))), 2)) + maxstar)';
                
                if ( isinf(temp2(j, part)) == 1 )
                    stop;
                end
                
            end
        end
        
        logpred(step1, check1(:, step1)==1) = logpred(step1, check1(:, step1)==1) + temp1(check1(:, step1)==1);
        logpred_t(step1, t + trainT + hsteps(step1) - 1, check1(:, step1)==1) = temp1(check1(:, step1)==1);
        
        for j = 1:p
            temp3 = temp2(j, check1(:, step1)==1);
            logpred_ind(step1, check1(:, step1)==1, j) = logpred_ind(step1, check1(:, step1)==1, j) + temp3;
            logpred_ind_t(j, step1, t + trainT + hsteps(step1) - 1, check1(:, step1)==1) = temp3;
        end
    end
end

output = struct('logpost', logpost, 'mu', mu, 'P', P, 'SuperP', SuperP, 'S', S, 'V', V, 'M', M, 'prob', prob, 'weight', weight, 'numbofcomps', it, 'logpred', logpred, 'logpred_t', logpred_t,  'logpred_ind', logpred_ind, 'logpred_ind_t', logpred_ind_t);
